@@ -65,12 +65,18 @@ get_match_type_str (enum match_type type)
 	switch (type) {
 	case MATCH_STRING:
 		return "string";
+	case MATCH_STRING_OUTOF:
+		return "string_outof";
 	case MATCH_INT:
 		return "int";
+	case MATCH_INT_OUTOF:
+		return "int_outof";
 	case MATCH_UINT64:
 		return "uint64";
 	case MATCH_BOOL:
 		return "bool";
+	case MATCH_DOUBLE:
+		return "double";
 	case MATCH_EXISTS:
 		return "exists";
 	case MATCH_EMPTY:
@@ -85,10 +91,16 @@ get_match_type_str (enum match_type type)
 		return "contains";
 	case MATCH_CONTAINS_NCASE:
 		return "contains_ncase";
+	case MATCH_CONTAINS_NOT:
+		return "contains_not";
+	case MATCH_CONTAINS_OUTOF:
+		return "contains_outof";
 	case MATCH_PREFIX:
 		return "prefix";
 	case MATCH_PREFIX_NCASE:
 		return "prefix_ncase";
+	case MATCH_PREFIX_OUTOF:
+		return "prefix_outof";
 	case MATCH_SUFFIX:
 		return "suffix";
 	case MATCH_SUFFIX_NCASE:
@@ -105,8 +117,6 @@ get_match_type_str (enum match_type type)
 		return "compare_ne";
 	case MATCH_UNKNOWN:
 		return "unknown match type";
-	case MATCH_CONTAINS_NOT:
-		return "contains_not";
 	}
 	return "invalid match type";
 }
@@ -269,7 +279,7 @@ handle_match (struct rule *rule, HalDevice *d)
 				   hal_device_get_udi (d),
 				   udi_to_check, sizeof (udi_to_check),
 				   prop_to_check, sizeof (prop_to_check))) {
-		HAL_ERROR (("Could not resolve keypath '%s' on udi '%s'", key, value));
+		/*HAL_ERROR (("Could not resolve keypath '%s' on udi '%s'", key, value));*/
 		return FALSE;
 	}
 
@@ -331,6 +341,17 @@ handle_match (struct rule *rule, HalDevice *d)
 		return TRUE;
 	}
 
+	case MATCH_DOUBLE:
+	{
+		double val = atof (value);
+
+		if (hal_device_property_get_type (d, prop_to_check) != HAL_PROPERTY_TYPE_DOUBLE)
+			return FALSE;
+		if (hal_device_property_get_double (d, prop_to_check) != val)
+			return FALSE;
+		return TRUE;
+	}
+
 	case MATCH_EXISTS:
 	{
 		dbus_bool_t should_exist = TRUE;
@@ -477,6 +498,81 @@ handle_match (struct rule *rule, HalDevice *d)
 			return !contains; /* rule->type_match == MATCH_CONTAINS_NOT  */
 		}
 	}
+	
+	case MATCH_CONTAINS_OUTOF:
+	case MATCH_PREFIX_OUTOF:
+	case MATCH_STRING_OUTOF:
+	{
+		dbus_bool_t contains = FALSE;
+
+		if (hal_device_has_property (d, prop_to_check) && value != NULL) {
+
+			if (hal_device_property_get_type (d, prop_to_check) == HAL_PROPERTY_TYPE_STRING) {
+				const char *haystack;
+				gchar **values;
+        			int i;
+
+                		values = g_strsplit (value, ";", 0);
+				
+				haystack = hal_device_property_get_string (d, prop_to_check);
+				if (haystack != NULL && values != NULL) {
+					for (i = 0; values [i] ; ++i) {
+						if (rule->type_match == MATCH_CONTAINS_OUTOF) {
+							if (strstr(haystack, values[i]) != NULL) {
+								contains = TRUE;
+								break;
+							}
+						}
+						else if (rule->type_match == MATCH_PREFIX_OUTOF) {
+							if (g_str_has_prefix (haystack, values[i])) {
+								contains = TRUE;
+								break;
+							}	
+						}
+						else if (rule->type_match == MATCH_STRING_OUTOF) {
+							if (strcmp (haystack, values[i]) == 0) {
+								contains = TRUE;
+								break;
+							}
+						}
+					}
+				}
+				g_strfreev (values);	
+			} 
+		}
+	
+		return contains;
+	}
+
+	case MATCH_INT_OUTOF:
+	{
+		dbus_bool_t contained = FALSE;
+
+		if (hal_device_has_property (d, prop_to_check) && value != NULL) {
+
+			if (hal_device_property_get_type (d, prop_to_check) == HAL_PROPERTY_TYPE_INT32) {
+				gchar **values;
+        			int i;
+				int to_check;
+
+                		values = g_strsplit (value, ";", 0);
+				to_check = hal_device_property_get_int (d, prop_to_check);				
+
+				if (values != NULL) {
+					for (i = 0; values [i] ; ++i) {
+                				if (to_check == strtol (values[i], NULL, 0)) {
+							contained = TRUE;
+							break;
+						}	
+					}
+				}
+				g_strfreev (values);	
+			} 
+		}
+	
+		return contained;
+	}
+
 
 	case MATCH_SIBLING_CONTAINS:
 	{
@@ -725,8 +821,39 @@ spawned_device_callouts_add_done (HalDevice *d, gpointer userdata1, gpointer use
 static gboolean
 handle_merge (struct rule *rule, HalDevice *d)
 {
-	const char *key = rule->key;
 	const char *value = (char *)RULES_PTR(rule->value_offset);
+	const char *key;
+
+	if (rule->rtype == RULE_MERGE || rule->rtype == RULE_APPEND || 
+	    rule->rtype == RULE_PREPEND || rule->rtype == RULE_ADDSET ) {
+		char udi_to_merge[HAL_PATH_MAX];
+        	char key_to_merge[HAL_PATH_MAX];
+
+		/* Resolve key paths like 'someudi/foo/bar/baz:prop.name' '@prop.here.is.an.udi:with.prop.name' */
+                if (!resolve_udiprop_path (rule->key, hal_device_get_udi (d),
+                                           udi_to_merge, sizeof (udi_to_merge),
+                                           key_to_merge, sizeof (key_to_merge))) {
+                        HAL_ERROR (("Could not resolve keypath '%s' on udi '%s'", rule->key, hal_device_get_udi (d)));
+			return FALSE;
+		} else {
+			key = g_strdup(key_to_merge);	
+
+			if (strcmp(hal_device_get_udi (d), udi_to_merge) != 0) {
+
+				d = hal_device_store_find (hald_get_gdl (), udi_to_merge);
+				if (d == NULL) {
+					d = hal_device_store_find (hald_get_tdl (), udi_to_merge);
+
+					if (d == NULL) {
+						HAL_ERROR (("Could not find device with udi '%s'", udi_to_merge));
+						return FALSE;
+					}
+				}
+			}
+		}
+	} else {
+		key = rule->key;
+	} 
 
 	if (rule->rtype == RULE_MERGE) {
 
@@ -738,7 +865,7 @@ handle_merge (struct rule *rule, HalDevice *d)
 
 			if (type == HAL_PROPERTY_TYPE_STRLIST || type == HAL_PROPERTY_TYPE_INVALID) {
 				hal_device_property_remove (d, key);
-				hal_device_property_strlist_append (d, key, value);
+				hal_device_property_strlist_append (d, key, value, FALSE);
 			}
 
 		} else if (rule->type_merge == MERGE_INT32) {
@@ -786,7 +913,7 @@ handle_merge (struct rule *rule, HalDevice *d)
 			HAL_ERROR (("unknown merge type (%u)", rule->type_merge));
 		}
 
-	} else if (rule->rtype == RULE_APPEND) {
+	} else if (rule->rtype == RULE_APPEND || rule->rtype == RULE_PREPEND) {
 		char buf[HAL_PATH_MAX];
 		char buf2[HAL_PATH_MAX];
 
@@ -798,66 +925,65 @@ handle_merge (struct rule *rule, HalDevice *d)
 		}
 
 		if (rule->type_merge == MERGE_STRLIST) {
-			hal_device_property_strlist_append (d, key, value);
-		} else {
+			if (rule->rtype == RULE_APPEND)
+				hal_device_property_strlist_append (d, key, value, FALSE);
+			else	/* RULE_PREPEND */ 
+				hal_device_property_strlist_prepend (d, key, value);
+		} else { 
 			const char *existing_string;
 
 			switch (rule->type_merge) {
 			case MERGE_STRING:
 				strncpy (buf, value, sizeof (buf));
 				break;
-
 			case MERGE_COPY_PROPERTY:
-				hal_device_property_get_as_string (d, value, buf, sizeof (buf));
-				break;
+			{
+				char udi_to_merge_from[HAL_PATH_MAX];
+				char prop_to_merge[HAL_PATH_MAX];
 
+				/* Resolve key paths like 'someudi/foo/bar/baz:prop.name'
+				 * '@prop.here.is.an.udi:with.prop.name'
+				 */
+				if (!resolve_udiprop_path (value,
+							   hal_device_get_udi (d),
+							   udi_to_merge_from, sizeof (udi_to_merge_from),
+							   prop_to_merge, sizeof (prop_to_merge))) {
+					HAL_ERROR (("Could not resolve keypath '%s' on udi '%s'", value, hal_device_get_udi (d)));
+				} else {
+					HalDevice *copyfrom;
+
+					copyfrom = hal_device_store_find (hald_get_gdl (), udi_to_merge_from);
+					if (copyfrom == NULL) {
+						copyfrom = hal_device_store_find (hald_get_tdl (), udi_to_merge_from);
+					}
+					if (copyfrom == NULL) {
+						HAL_ERROR (("Could not find device with udi '%s'", udi_to_merge_from));
+					} else {
+						hal_device_property_get_as_string (copyfrom, prop_to_merge, buf, sizeof (buf));
+					}
+				}
+				break;
+			}
 			default:
 				break;
 			}
-
+		
 			existing_string = hal_device_property_get_string (d, key);
 			if (existing_string != NULL) {
-				strncpy (buf2, existing_string, sizeof (buf2));
-				strncat (buf2, buf, sizeof (buf2) - strlen(buf2));
-			} else
-				strncpy (buf2, buf, sizeof (buf2));
-			hal_device_property_set_string (d, key, buf2);
-		}
-
-	} else if (rule->rtype == RULE_PREPEND) {
-		char buf[HAL_PATH_MAX];
-		char buf2[HAL_PATH_MAX];
-
-		if (hal_device_property_get_type (d, key) != HAL_PROPERTY_TYPE_STRING &&
-		    hal_device_property_get_type (d, key) != HAL_PROPERTY_TYPE_STRLIST &&
-		    hal_device_property_get_type (d, key) != HAL_PROPERTY_TYPE_INVALID) {
-			HAL_ERROR (("invalid key type"));
-			return FALSE;
-		}
-
-		if (rule->type_merge == MERGE_STRLIST) {
-			hal_device_property_strlist_prepend (d, key, value);
-		} else {
-			const char *existing_string;
-
-			if (rule->type_merge == MERGE_STRING) {
-				strncpy (buf, value, sizeof (buf));
-
-			} else if (rule->type_merge == MERGE_COPY_PROPERTY) {
-				hal_device_property_get_as_string (d, value, buf, sizeof (buf));
-
-			}
-
-			existing_string = hal_device_property_get_string (d, key);
-			if (existing_string != NULL) {
-				strncpy (buf2, buf, sizeof (buf2));
-				strncat (buf2, existing_string, sizeof (buf2) - strlen(buf2));
+				if (rule->rtype == RULE_APPEND) {
+					strncpy (buf2, existing_string, sizeof (buf2));
+					strncat (buf2, buf, sizeof (buf2) - strlen(buf2));
+				} else { /* RULE_PREPEND */
+					strncpy (buf2, buf, sizeof (buf2));
+					strncat (buf2, existing_string, sizeof (buf2) - strlen(buf2));
+				}
 			} else {
 				strncpy (buf2, buf, sizeof (buf2));
 			}
-			hal_device_property_set_string (d, key, buf2);
-		}
 
+			hal_device_property_set_string (d, key, buf2);
+		
+		}
 	} else if (rule->rtype == RULE_ADDSET) {
 
 		if (hal_device_property_get_type (d, key) != HAL_PROPERTY_TYPE_STRLIST &&
@@ -868,7 +994,7 @@ handle_merge (struct rule *rule, HalDevice *d)
 
                 if (!hal_device_has_property (d, key) ||
                     !hal_device_property_strlist_contains (d, key, value)) {
-                        hal_device_property_strlist_append (d, key, value);
+                        hal_device_property_strlist_append (d, key, value, FALSE);
                 }
 
 	} else if (rule->rtype == RULE_REMOVE) {
@@ -988,19 +1114,14 @@ rules_match_and_merge_device (void *fdi_rules_list, HalDevice *d)
 	}
 }
 
-static gboolean fdi_cache_invalidated = FALSE;
-
-
 /* merge the device info type, either preprobe, info or policy */
 gboolean
 di_search_and_merge (HalDevice *d, DeviceInfoType type){
 	struct cache_header *header = (struct cache_header*) RULES_PTR(0);
 
-	if (fdi_cache_invalidated) {
-		/* make sure our fdi rule cache is up to date */
-		if (di_cache_coherency_check ())
-			di_rules_init ();
-		fdi_cache_invalidated = FALSE;
+        /* make sure our fdi rule cache is up to date */
+        if (di_cache_coherency_check (FALSE)) {
+                di_rules_init ();
 	}
 
 	switch (type) {
@@ -1043,12 +1164,3 @@ di_search_and_merge (HalDevice *d, DeviceInfoType type){
 
 	return TRUE;
 }
-
-/* Called by OS specific code to tell that the fdi cache is invalid */
-void 
-osspec_fdi_cache_invalid (void)
-{
-	HAL_INFO (("invalidating fdi cache"));
-	fdi_cache_invalidated = TRUE;
-}
-
