@@ -5,6 +5,7 @@
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  * Copyright (C) 2005 Danny Kukawka, <danny.kukawka@web.de>
+ * Copyright (C) 2007 Codethink Ltd. Author Rob Taylor <rob.taylor@codethink.co.uk>
  *
  * Licensed under the Academic Free License version 2.1
  *
@@ -30,6 +31,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+
+#ifdef HAVE_MALLOPT
+#include <malloc.h>
+#endif
+
 #include <string.h>
 #include <unistd.h>
 #include <getopt.h>
@@ -43,7 +49,6 @@
 #include <syslog.h>
 #include <sys/time.h>
 #include <sys/resource.h>
-#include <malloc.h>
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -119,8 +124,33 @@ gdl_store_changed (HalDeviceStore *store, HalDevice *device,
 					    command_line, hal_device_get_udi(device)));
 			}
 		}
+		for (hal_device_property_strlist_iter_init (device, "info.addons.singleton", &iter);
+		     hal_device_property_strlist_iter_is_valid (&iter);
+		     hal_device_property_strlist_iter_next (&iter)) {
+			const gchar *command_line;
+
+			command_line = hal_device_property_strlist_iter_get_value (&iter);
+
+			if (hald_singleton_device_added (command_line, device))
+				hal_device_inc_num_addons (device);
+			else
+				HAL_ERROR(("Couldn't add device to singleton"));
+		}
+
 	} else {
+		HalDeviceStrListIter iter;
+
 		HAL_INFO (("Removed device from GDL; udi=%s", hal_device_get_udi(device)));
+		for (hal_device_property_strlist_iter_init (device, "info.addons.singleton", &iter);
+		     hal_device_property_strlist_iter_is_valid (&iter);
+		     hal_device_property_strlist_iter_next (&iter)) {
+			const gchar *command_line;
+
+			command_line = hal_device_property_strlist_iter_get_value (&iter);
+
+			hald_singleton_device_removed (command_line, device);
+		}
+
 		hald_runner_kill_device(device);
 	}
 
@@ -265,6 +295,10 @@ dbus_bool_t hald_is_verbose = FALSE;
 dbus_bool_t hald_use_syslog = FALSE;
 dbus_bool_t hald_debug_exit_after_probing = FALSE;
 
+#ifdef HAVE_POLKIT
+PolKitContext *pk_context;
+#endif
+
 static int sigterm_unix_signal_pipe_fds[2];
 static GIOChannel *sigterm_iochn;
 
@@ -382,6 +416,75 @@ out:
 
 /*--------------------------------------------------------------------------------------------------*/
 
+#ifdef HAVE_POLKIT
+
+static gboolean
+_polkit_io_watch_have_data (GIOChannel *channel, GIOCondition condition, gpointer user_data)
+{
+        int fd;
+        PolKitContext *pk_context = user_data;
+        fd = g_io_channel_unix_get_fd (channel);
+        polkit_context_io_func (pk_context, fd);
+        return TRUE;
+}
+
+static int 
+_polkit_io_add_watch (PolKitContext *pk_context, int fd)
+{
+        guint id = 0;
+        GIOChannel *channel;
+        channel = g_io_channel_unix_new (fd);
+        if (channel == NULL)
+                goto out;
+        id = g_io_add_watch (channel, G_IO_IN, _polkit_io_watch_have_data, pk_context);
+        if (id == 0) {
+                g_io_channel_unref (channel);
+                goto out;
+        }
+        g_io_channel_unref (channel);
+out:
+        return id;
+}
+
+static void 
+_polkit_io_remove_watch (PolKitContext *pk_context, int watch_id)
+{
+        g_source_remove (watch_id);
+}
+
+static guint _polkit_cooloff_timer = 0;
+
+static gboolean
+_polkit_config_really_changed (gpointer user_data)
+{
+        HAL_INFO (("Acting on PolicyKit config change"));
+        _polkit_cooloff_timer = 0;
+        reconfigure_all_policy ();
+        return FALSE;
+}
+
+static void
+_polkit_config_changed_cb (PolKitContext *pk_context, gpointer user_data)
+{
+        HAL_INFO (("PolicyKit reports that the config have changed; starting cool-off timer"));
+
+        /* Start a cool-off timer since we get a lot of events within
+         * a short time-frame...
+         */
+
+        if (_polkit_cooloff_timer > 0) {
+                /* cancel old cool-off timer */
+                g_source_remove (_polkit_cooloff_timer);
+                HAL_INFO (("restarting cool-off timer"));
+        }
+        _polkit_cooloff_timer = g_timeout_add (1000, /* one second... */
+                                               _polkit_config_really_changed,
+                                               NULL);
+}
+
+#endif /* HAVE_POLKIT */
+
+
 /**  
  *  main:
  *  @argc:               Number of arguments
@@ -397,7 +500,9 @@ main (int argc, char *argv[])
 	guint sigterm_iochn_listener_source_id;
 	char *path;
 	char newpath[512];
-
+#ifdef HAVE_POLKIT
+        PolKitError *p_error;
+#endif
 	openlog ("hald", LOG_PID, LOG_DAEMON);
 
 #ifdef HAVE_MALLOPT
@@ -632,6 +737,19 @@ main (int argc, char *argv[])
 	/* initialize privileged operating system specific parts */
 	osspec_privileged_init ();
 
+#ifdef HAVE_POLKIT
+        p_error = NULL;
+        pk_context = polkit_context_new ();
+        if (pk_context == NULL)
+                DIE (("Could not create PolicyKit context"));
+        polkit_context_set_config_changed (pk_context,
+                                           _polkit_config_changed_cb,
+                                           NULL);
+        polkit_context_set_io_watch_functions (pk_context, _polkit_io_add_watch, _polkit_io_remove_watch);
+        if (!polkit_context_init (pk_context, &p_error))
+                DIE (("Could not init PolicyKit context: %s", polkit_error_get_error_message (p_error)));
+#endif
+
 	/* sometimes we don't want to drop root privs, for instance
 	   if we are profiling memory usage */
 	if (opt_retain_privileges == FALSE) {
@@ -640,8 +758,8 @@ main (int argc, char *argv[])
 
 	hald_is_initialising = TRUE;
 
-	/* make sure our fdi rule cache is up to date */
-	di_cache_coherency_check();
+	/* make sure our fdi rule cache is up to date and setup file monitoring */
+	di_cache_coherency_check(TRUE);
 
 	/* initialize operating system specific parts */
 	osspec_init ();
