@@ -4,6 +4,7 @@
  * hald_runner.c - Interface to the hal runner helper daemon
  *
  * Copyright (C) 2006 Sjoerd Simons, <sjoerd@luon.net>
+ * Copyright (C) 2007 Codethink Ltd. Author Rob Taylor <rob.taylor@codethink.co.uk>
  *
  * Licensed under the Academic Free License version 2.1
  *
@@ -65,6 +66,7 @@ typedef struct {
 	GPid pid;
 	HalDevice *device;
 	HalRunTerminatedCB cb;
+	gboolean is_singleton;
 	gpointer data1;
 	gpointer data2;
 } RunningProcess;
@@ -341,10 +343,12 @@ add_property_to_msg (HalDevice * device,
 static void
 add_env (DBusMessageIter * iter, const gchar * key, const gchar * value)
 {
-	gchar *env;
-	env = g_strdup_printf ("%s=%s", key, value);
-	dbus_message_iter_append_basic (iter, DBUS_TYPE_STRING, &env);
-	g_free (env);
+        if (value != NULL) {
+                gchar *env;
+                env = g_strdup_printf ("%s=%s", key, value);
+                dbus_message_iter_append_basic (iter, DBUS_TYPE_STRING, &env);
+                g_free (env);
+        }
 }
 
 static void
@@ -364,10 +368,17 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 	if (hald_use_syslog) {
 		add_env (iter, "HALD_USE_SYSLOG", "1");
 	}
-	add_env (iter, "UDI", udi);
+	if (udi)
+		add_env (iter, "UDI", udi);
 	add_env (iter, "HALD_DIRECT_ADDR", hald_dbus_local_server_addr ());
+
+        /* I'm sure it would be easy to remove use of getenv(3) to add these variables... */
+
+	add_env (iter, "LD_LIBRARY_PATH", getenv ("LD_LIBRARY_PATH"));
 #ifdef HAVE_POLKIT
 	add_env (iter, "HAVE_POLKIT", "1");
+        add_env (iter, "POLKIT_POLICY_DIR", getenv ("POLKIT_POLICY_DIR"));
+        add_env (iter, "POLKIT_DEBUG", getenv ("POLKIT_DEBUG"));
 #endif
 
 
@@ -379,16 +390,21 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 		GString *seats_string;
 		char *s;
 		char *p;
+                int num_seats;
+                int num_sessions_total;
 
 		seats_string = g_string_new ("");
 
 		seats = ck_tracker_get_seats (ck_tracker);
+                num_seats = g_slist_length (seats);
+                num_sessions_total = 0;
 		for (i = seats; i != NULL; i = g_slist_next (i)) {
 			GSList *j;
 			CKSeat *seat;
 			GSList *sessions;
 			const char *seat_id;
 			GString *sessions_string;
+                        int num_sessions;
 
 			sessions_string = g_string_new ("");
 
@@ -404,9 +420,12 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 			g_string_append (seats_string, seat_id);
 
 			sessions = ck_seat_get_sessions (seat);
+                        num_sessions = g_slist_length (sessions);
+                        num_sessions_total += num_sessions;
 			for (j = sessions; j != NULL; j = g_slist_next (j)) {
 				CKSession *session;
 				const char *session_id;
+                                const char *q;
 
 				session = j->data;
 				/* basename again; e.g. Session1 rather than /org/freedesktop/ConsoleKit/Session1 */
@@ -417,13 +436,17 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 				}
 				g_string_append (sessions_string, session_id);
 
-				/* for each Session, export IS_ACTIVE and UID 
+				/* for each Session, export IS_ACTIVE, UID, IS_LOCAL, HOSTNAME
 				 *
+                                 * CK_SESSION_SEAT_Session2=Seat1
 				 * CK_SESSION_IS_ACTIVE_Session2=true|false
 				 * CK_SESSION_UID_Session2=501
 				 * CK_SESSION_IS_LOCAL_Session2=true|false
 				 * CK_SESSION_HOSTNAME_Session2=192.168.1.112
 				 */
+				s = g_strdup_printf ("CK_SESSION_SEAT_%s", session_id);
+				add_env (iter, s, seat_id);
+				g_free (s);
 				s = g_strdup_printf ("CK_SESSION_IS_ACTIVE_%s", session_id);
 				add_env (iter, s, ck_session_is_active (session) ? "true" : "false");
 				g_free (s);
@@ -435,22 +458,30 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 				s = g_strdup_printf ("CK_SESSION_IS_LOCAL_%s", session_id);
 				add_env (iter, s, ck_session_is_local (session) ? "true" : "false");
 				g_free (s);
-				s = g_strdup_printf ("CK_SESSION_HOSTNAME_%s", session_id);
-				p = g_strdup_printf ("%s", ck_session_get_hostname (session));
-				add_env (iter, s, p);
-				g_free (s);
-				g_free (p);
+				q = ck_session_get_hostname (session);
+                                if (q != NULL && strlen (q) > 0) {
+                                        s = g_strdup_printf ("CK_SESSION_HOSTNAME_%s", session_id);
+                                        add_env (iter, s, q);
+                                        g_free (s);
+                                }
 			}
 
 			/* for each Seat, export sessions on each seat 
 			 *
 			 * CK_SEAT_Seat1=Session1 Session3 Session7
+                         * CK_SEAT_NUM_SESSIONS_Seat1=3
 			 */
 			s = g_strdup_printf ("CK_SEAT_%s", seat_id);
 			add_env (iter, s, sessions_string->str);
 			g_string_free (sessions_string, TRUE);
 			g_free (s);
 
+			s = g_strdup_printf ("CK_SEAT_NUM_SESSIONS_%s", seat_id);
+                        p = g_strdup_printf ("%d", num_sessions);
+                        add_env (iter, s, p);
+                        g_free (p);
+			g_free (s);
+
 		}
 
 		/* Export all detected seats 
@@ -459,6 +490,23 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 		 */
 		add_env (iter, "CK_SEATS", seats_string->str);
 		g_string_free (seats_string, TRUE);
+
+		/* Export total number of sessions
+		 *
+		 * CK_NUM_SESSIONS=5
+		 */
+                p = g_strdup_printf ("%d", num_sessions_total);
+                add_env (iter, "CK_NUM_SESSIONS", p);
+                g_free (p);
+
+		/* Export number of seats
+		 *
+		 * CK_NUM_SEATS=5
+		 */
+                p = g_strdup_printf ("%d", num_seats);
+                add_env (iter, "CK_NUM_SEATS", p);
+                g_free (p);
+
 	}
 #endif /* HAVE_CONKIT */
 
@@ -514,11 +562,9 @@ add_command (DBusMessageIter * iter, const gchar * command_line)
 	return TRUE;
 }
 
-static gboolean
-add_first_part (DBusMessageIter * iter, HalDevice * device,
-		const gchar * command_line, char **extra_env)
+static void
+add_udi (DBusMessageIter * iter, HalDevice * device)
 {
-	DBusMessageIter array_iter;
 	const char *udi;
 
 	if (device != NULL)
@@ -527,7 +573,13 @@ add_first_part (DBusMessageIter * iter, HalDevice * device,
 		udi = "";
 
 	dbus_message_iter_append_basic (iter, DBUS_TYPE_STRING, &udi);
+}
 
+static gboolean
+add_environment (DBusMessageIter * iter, HalDevice * device,
+		const gchar * command_line, char **extra_env)
+{
+	DBusMessageIter array_iter;
 	dbus_message_iter_open_container (iter,
 					  DBUS_TYPE_ARRAY,
 					  DBUS_TYPE_STRING_AS_STRING,
@@ -535,7 +587,7 @@ add_first_part (DBusMessageIter * iter, HalDevice * device,
 	if (device != NULL)
 		hal_device_property_foreach (device, add_property_to_msg,
 					     &array_iter);
-	add_basic_env (&array_iter, udi);
+	add_basic_env (&array_iter, device ? hal_device_get_udi (device): NULL);
 	add_extra_env (&array_iter, extra_env);
 	dbus_message_iter_close_container (iter, &array_iter);
 
@@ -546,31 +598,37 @@ add_first_part (DBusMessageIter * iter, HalDevice * device,
 }
 
 /* Start a helper, returns true on a successfull start */
-gboolean
-hald_runner_start (HalDevice * device, const gchar * command_line,
-		   char **extra_env, HalRunTerminatedCB cb, gpointer data1,
-		   gpointer data2)
+static gboolean
+runner_start (HalDevice * device, const gchar * command_line,
+	      char **extra_env, gboolean singleton,
+	      HalRunTerminatedCB cb, gpointer data1, gpointer data2)
 {
 	DBusMessage *msg, *reply;
-	DBusError err;
+	DBusError error;
 	DBusMessageIter iter;
 
-	dbus_error_init (&err);
+	dbus_error_init (&error);
 	msg = dbus_message_new_method_call ("org.freedesktop.HalRunner",
 					    "/org/freedesktop/HalRunner",
 					    "org.freedesktop.HalRunner",
-					    "Start");
+					    singleton ? "StartSingleton" : "Start");
 	if (msg == NULL)
 		DIE (("No memory"));
 	dbus_message_iter_init_append (msg, &iter);
 
-	if (!add_first_part (&iter, device, command_line, extra_env))
+	if (!singleton)
+		add_udi (&iter, device);
+
+	if (!add_environment (&iter, device, command_line, extra_env)) {
+		HAL_ERROR (("Error adding environment, device =%p, singleton=%d",
+			device, singleton));
 		goto error;
+	}
 
 	/* Wait for the reply, should be almost instantanious */
 	reply =
 	    dbus_connection_send_with_reply_and_block (runner_connection,
-						       msg, -1, &err);
+						       msg, -1, &error);
 	if (reply) {
 		gboolean ret =
 		    (dbus_message_get_type (reply) ==
@@ -578,7 +636,7 @@ hald_runner_start (HalDevice * device, const gchar * command_line,
 
 		if (ret) {
 			dbus_int64_t pid_from_runner;
-			if (dbus_message_get_args (reply, &err,
+			if (dbus_message_get_args (reply, &error,
 						   DBUS_TYPE_INT64,
 						   &pid_from_runner,
 						   DBUS_TYPE_INVALID)) {
@@ -587,7 +645,11 @@ hald_runner_start (HalDevice * device, const gchar * command_line,
 					rp = g_new0 (RunningProcess, 1);
 					rp->pid = (GPid) pid_from_runner;
 					rp->cb = cb;
-					rp->device = device;
+					rp->is_singleton = singleton;
+					if (singleton)
+						rp->device = NULL;
+					else
+						rp->device = device;
 					rp->data1 = data1;
 					rp->data2 = data2;
 
@@ -604,6 +666,10 @@ hald_runner_start (HalDevice * device, const gchar * command_line,
 		dbus_message_unref (reply);
 		dbus_message_unref (msg);
 		return ret;
+	} else {
+		if (dbus_error_is_set (&error)) {
+			HAL_ERROR (("Error running '%s': %s: %s", command_line, error.name, error.message));
+		}
 	}
 
       error:
@@ -611,6 +677,23 @@ hald_runner_start (HalDevice * device, const gchar * command_line,
 	return FALSE;
 }
 
+gboolean
+hald_runner_start (HalDevice * device, const gchar * command_line,
+		   char **extra_env, HalRunTerminatedCB cb,
+		   gpointer data1, gpointer data2)
+{
+	return runner_start (device, command_line, extra_env, FALSE, cb, data1, data2);
+}
+
+gboolean
+hald_runner_start_singleton (const gchar * command_line,
+			     char **extra_env, HalRunTerminatedCB cb,
+			     gpointer data1, gpointer data2)
+{
+	return runner_start (NULL, command_line, extra_env, TRUE, cb, data1, data2);
+}
+
+
 static void
 process_reply (DBusMessage *m, HelperData *hb)
 {
@@ -714,7 +797,8 @@ hald_runner_run_method (HalDevice * device,
 		DIE (("No memory"));
 	dbus_message_iter_init_append (msg, &iter);
 
-	if (!add_first_part (&iter, device, command_line, extra_env))
+	add_udi (&iter, device);
+	if (!add_environment (&iter, device, command_line, extra_env))
 		goto error;
 
 	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input);
@@ -783,7 +867,8 @@ hald_runner_run_sync (HalDevice * device,
 		DIE (("No memory"));
 	dbus_message_iter_init_append (msg, &iter);
 
-	if (!add_first_part (&iter, device, command_line, extra_env))
+	add_udi (&iter, device);
+	if (!add_environment (&iter, device, command_line, extra_env))
 		goto error;
 
 	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input);
@@ -853,13 +938,11 @@ hald_runner_kill_device (HalDevice * device)
 }
 
 void
-hald_runner_kill_all (HalDevice * device)
+hald_runner_kill_all ()
 {
 	DBusMessage *msg, *reply;
 	DBusError err;
 
-	running_processes_remove_device (device);
-
 	msg = dbus_message_new_method_call ("org.freedesktop.HalRunner",
 					    "/org/freedesktop/HalRunner",
 					    "org.freedesktop.HalRunner",
