@@ -229,10 +229,10 @@ battery_refresh (HalDevice *d, APMDevHandler *handler)
 		hal_device_property_set_bool (d, "battery.rechargeable.is_discharging", is_discharging);
 
 		/* set the percentage charge, easy. */
-		remaining_percentage = util_compute_percentage_charge (hal_device_get_udi (d), i.battery_percentage, 100);
+		remaining_percentage = i.battery_percentage;
 	
 		/* Only set keys if no error (signified with negative return value) */
-		if (remaining_percentage > 0)
+		if (remaining_percentage >= 0 && remaining_percentage <= 100)
 			hal_device_property_set_int (d, "battery.charge_level.percentage", remaining_percentage);
 		else
 			hal_device_property_remove (d, "battery.charge_level.percentage");
@@ -278,7 +278,6 @@ apm_synthesize_hotplug_events (void)
 {
 	gboolean ret;
 	HalDevice *computer;
-	gchar path[HAL_PATH_MAX];
 	HotplugEvent *hotplug_event;
 
 	ret = FALSE;
@@ -297,18 +296,16 @@ apm_synthesize_hotplug_events (void)
 	/* Set appropriate properties on the computer object */
 	hal_device_property_set_string (computer, "power_management.type", "apm");
 
-	snprintf (path, sizeof (path), "%s/apm", get_hal_proc_path ());
-
 	hotplug_event = g_new0 (HotplugEvent, 1);
 	hotplug_event->action = HOTPLUG_ACTION_ADD;
 	hotplug_event->type = HOTPLUG_EVENT_APM;
-	g_strlcpy (hotplug_event->apm.apm_path, path, sizeof (hotplug_event->apm.apm_path));
+	g_strlcpy (hotplug_event->apm.apm_path, "/proc/apm", sizeof (hotplug_event->apm.apm_path));
 	hotplug_event->apm.apm_type = APM_TYPE_BATTERY;
 	hotplug_event_enqueue (hotplug_event);
 
 	hotplug_event = g_new0 (HotplugEvent, 1);
 	hotplug_event->type = HOTPLUG_EVENT_APM;
-	g_strlcpy (hotplug_event->apm.apm_path, path, sizeof (hotplug_event->apm.apm_path));
+	g_strlcpy (hotplug_event->apm.apm_path, "/proc/apm", sizeof (hotplug_event->apm.apm_path));
 	hotplug_event->apm.apm_type = APM_TYPE_AC_ADAPTER;
 	hotplug_event_enqueue (hotplug_event);
 
