@@ -67,16 +67,21 @@ blockdev_compute_udi (HalDevice *d)
 	if (hal_device_property_get_bool (d, "block.is_volume")) {
 		const char *label;
 		const char *uuid;
+		char *volumelabel;
 
 		label = hal_device_property_get_string (d, "volume.label");
+	        /* replace '/'  to avoid trouble if the string get part of the UDI see fd.o #11401 */
+		volumelabel = g_strdup(label);
+	        volumelabel = g_strdelimit (volumelabel, "/", '_');
+
 		uuid = hal_device_property_get_string (d, "volume.uuid");
 
 		if (uuid != NULL && strlen (uuid) > 0) {
 			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
 					      "/org/freedesktop/Hal/devices/volume_uuid_%s", uuid);
-		} else if (label != NULL && strlen (label) > 0) {
+		} else if (volumelabel != NULL && strlen (volumelabel) > 0) {
 			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-					      "/org/freedesktop/Hal/devices/volume_label_%s", label);
+					      "/org/freedesktop/Hal/devices/volume_label_%s", volumelabel);
 		} else if (hal_device_property_get_bool(d, "volume.is_disc") &&
 			   hal_device_property_get_bool(d, "volume.disc.is_blank")) {
 			/* this should be a empty CD/DVD */
@@ -90,6 +95,7 @@ blockdev_compute_udi (HalDevice *d)
 					      hal_device_property_get_int (d, "volume.partition.number"),
 					      hal_device_property_get_uint64 (d, "volume.size"));
 		}
+		g_free(volumelabel);
 	} else {
 		const char *model;
 		const char *serial;
@@ -165,14 +171,11 @@ blockdev_refresh_mount_state (HalDevice *d)
 	struct mntent mnt;
 	struct mntent *mnte;
 	char buf[1024];
-	unsigned int major;
-	unsigned int minor;
-	dev_t devt = makedev(0, 0);
 	GSList *volumes = NULL;
 	GSList *volume;
 
 	/* open /proc/mounts */
-	g_snprintf (buf, sizeof (buf), "%s/mounts", get_hal_proc_path ());
+	g_snprintf (buf, sizeof (buf), "%s/mounts", "/proc");
 	if ((f = setmntent (buf, "r")) == NULL) {
 		HAL_ERROR (("Could not open /proc/mounts"));
 		return;
@@ -189,43 +192,53 @@ blockdev_refresh_mount_state (HalDevice *d)
 	/* loop over /proc/mounts */
 	while ((mnte = getmntent_r (f, &mnt, buf, sizeof(buf))) != NULL) {
 		struct stat statbuf;
-		gboolean use_device_name_for_match;
+		dev_t devt;
 
-		/*HAL_INFO ((" * /proc/mounts contain dev %s - type %s", mnt.mnt_fsname, mnt.mnt_type));*/
+		/* HAL_INFO ((" * /proc/mounts contain dev %s - type %s", mnt.mnt_fsname, mnt.mnt_type)); */
 
-		/* If this is a nfs mount (fstype == 'nfs') ignore the mount. Reason:
-		 *  1. we don't list nfs devices in HAL
-		 *  2. more problematic: stat on mountpoints with 'stale nfs handle' never come
-		 *     back and block complete HAL and all applications using HAL fail.
+		/* We don't handle nfs mounts in HAL and stat() on mountpoints,
+		 * and we would block on 'stale nfs handle'.
 		 */
 		if (strcmp(mnt.mnt_type, "nfs") == 0)
 			continue;
 
-		use_device_name_for_match = FALSE;
-		/* get major:minor of special device file */
-		if (stat (mnt.mnt_fsname, &statbuf) != 0) {
-			/* DING DING DING... device node may have been deleted by udev 
-			 * but device is still mounted and we haven't processed the uevent
-			 * for that deletion from udev.. 
-			 * 
-			 * So in this case... fall back to comparing on device names 
-			 * rather than pretending the device is not mounted as that's
-			 * what will happen if we just skip this /proc/mounts entry.
-			 *
-			 * The reason it's nicer to compare on major:minor is that
-			 * /proc/mounts is broken - it contains the *device name*
-			 * passed to mount(2) which in some cases may be a symlink
-			 * (on many distros it's common to see /proc/mounts contain
-			 * /dev/root as the device for /). Sigh...
-			 */
-			use_device_name_for_match = TRUE;
-		} else {
+		/* skip plain names, we look for device nodes */
+		if (mnt.mnt_fsname[0] != '/')
+			continue;
+
+		/*
+		 * We can't just stat() the mountpoint, because it breaks all sorts
+		 * non-disk filesystems. So assume, that the names in /proc/mounts
+		 * are existing device-files used to mount the filesystem.
+		 */
+		devt = makedev(0, 0);
+		if (stat (mnt.mnt_fsname, &statbuf) == 0) {
+			/* not a device node */
 			if (major (statbuf.st_rdev) == 0)
 				continue;
+
+			/* found major/minor */
+			devt = statbuf.st_rdev;
+		} else {
+			/* The root filesystem may be mounted by a device name that doesn't
+			 * exist in the real root, like /dev/root, which the kernel uses
+			 * internally, when no initramfs image is used. For "/", it is safe
+			 * to get the major/minor by stat()'ing the mount-point.
+			 */
+			if (strcmp (mnt.mnt_dir, "/") == 0 && stat ("/", &statbuf) == 0)
+				devt = statbuf.st_dev;
+
+			/* DING DING DING... the device-node may not exist, or is
+			 * already deleted, but the device may be still mounted.
+			 *
+			 * We will fall back to looking up the device-name, instead
+			 * of using major/minor.
+		 	 */
 		}
 
-		/*HAL_INFO (("* found mounts dev %s (%i:%i)", mnt.mnt_fsname, major (statbuf.st_rdev), minor (statbuf.st_rdev)));*/
-		/* match against all hal volumes */
+		/* HAL_INFO (("* found mounts dev %s (%i:%i)", mnt.mnt_fsname,
+			   major (devt), minor (devt))); */
+
 		for (volume = volumes; volume != NULL; volume = g_slist_next (volume)) {
 			HalDevice *dev;
 			gboolean is_match;
@@ -233,7 +246,8 @@ blockdev_refresh_mount_state (HalDevice *d)
 			is_match = FALSE;
 			dev = HAL_DEVICE (volume->data);
 
-			if (use_device_name_for_match) {
+			/* lookup dev_t or devname of known hal devices */
+			if (major (devt) == 0) {
 				const char *device_name;
 
 				device_name = hal_device_property_get_string (dev, "block.device");
@@ -243,14 +257,16 @@ blockdev_refresh_mount_state (HalDevice *d)
 				if (strcmp (device_name, mnt.mnt_fsname) == 0)
 					is_match = TRUE;
 			} else {
-				major = hal_device_property_get_int (dev, "block.major");
-				if (major == 0)
-					continue;
-				minor = hal_device_property_get_int (dev, "block.minor");
-				devt = makedev (major, minor);
-				/*HAL_INFO (("  match %s (%i:%i)", hal_device_get_udi (dev), major, minor));*/
+				unsigned int majornum;
+				unsigned int minornum;
 
-				if (statbuf.st_rdev == devt)
+				majornum = hal_device_property_get_int (dev, "block.major");
+				if (majornum == 0)
+					continue;
+				minornum = hal_device_property_get_int (dev, "block.minor");
+				/* HAL_INFO (("  match %s (%i:%i)", hal_device_get_udi (dev), majornum, minornum)); */
+
+				if (majornum == major (devt) && minornum == minor (devt))
 					is_match = TRUE;
 			}
 
@@ -262,9 +278,8 @@ blockdev_refresh_mount_state (HalDevice *d)
 							      hasmntopt (&mnt, MNTOPT_RO) ? TRUE : FALSE);
 				hal_device_property_set_string (dev, "volume.mount_point", mnt.mnt_dir);
 				device_property_atomic_update_end ();
-				/*HAL_INFO (("  set %s to be mounted at %s (%s)",
-				  hal_device_get_udi (dev), mnt.mnt_dir,
-					   hasmntopt (&mnt, MNTOPT_RO) ? "ro" : "rw"));*/
+				/* HAL_INFO (("  set %s to be mounted at %s (%s)", hal_device_get_udi (dev),
+					   mnt.mnt_dir, hasmntopt (&mnt, MNTOPT_RO) ? "ro" : "rw")); */
 				volumes = g_slist_delete_link (volumes, volume);
 				break;
 			}
@@ -626,6 +641,198 @@ out:
 	return f;
 }
 
+static gboolean
+refresh_md_state (HalDevice *d);
+
+static gboolean
+md_check_sync_timeout (gpointer user_data)
+{
+        HalDevice *d;
+        char *sysfs_path = (char *) user_data;
+
+        HAL_INFO (("In md_check_sync_timeout for sysfs path %s", sysfs_path));
+
+        d = hal_device_store_match_key_value_string (hald_get_gdl (), 
+                                                     "storage.linux_raid.sysfs_path", 
+                                                     sysfs_path);
+        if (d == NULL)
+                d = hal_device_store_match_key_value_string (hald_get_tdl (), 
+                                                             "storage.linux_raid.sysfs_path", 
+                                                             sysfs_path);
+        if (d == NULL) {
+                HAL_WARNING (("Cannot find md device with sysfs path '%s'", sysfs_path));
+                goto out;
+        }
+
+        refresh_md_state (d);
+        
+out:
+        g_free (sysfs_path);
+        return FALSE;
+}
+
+static gboolean
+refresh_md_state (HalDevice *d)
+{
+        int n;
+        char *sync_action;
+        int num_components;
+        gboolean ret;
+        const char *sysfs_path;
+
+        ret = FALSE;
+
+        sysfs_path = hal_device_property_get_string (d, "storage.linux_raid.sysfs_path");
+        if (sysfs_path == NULL) {
+                HAL_WARNING (("Cannot get sysfs_path for udi %s", hal_device_get_udi (d)));
+                goto error;
+        }
+
+        HAL_INFO (("In refresh_md_state() for '%s'", sysfs_path));
+
+        sync_action = hal_util_get_string_from_file (sysfs_path, "md/sync_action");
+        if (sync_action == NULL) {
+                HAL_WARNING (("Cannot get sync_action for %s", sysfs_path));
+                goto error;
+        }
+        if (strcmp (sync_action, "idle") == 0) {
+                hal_device_property_set_bool (d, "storage.linux_raid.is_syncing", FALSE);
+		hal_device_property_remove (d, "storage.linux_raid.sync.action");
+		hal_device_property_remove (d, "storage.linux_raid.sync.speed");
+		hal_device_property_remove (d, "storage.linux_raid.sync.progress");
+        } else {
+                int speed;
+                char *str_completed;
+
+                hal_device_property_set_bool (d, "storage.linux_raid.is_syncing", TRUE);
+
+                hal_device_property_set_string (d, "storage.linux_raid.sync.action", sync_action);
+
+		if (!hal_util_get_int_from_file (sysfs_path, "md/sync_speed", &speed, 10)) {
+                        HAL_WARNING (("Cannot get sync_speed for %s", sysfs_path));
+                } else {
+                        hal_device_property_set_uint64 (d, "storage.linux_raid.sync.speed", speed);
+                }
+
+
+                if ((str_completed = hal_util_get_string_from_file (sysfs_path, "md/sync_completed")) == NULL) {
+                        HAL_WARNING (("Cannot get sync_completed for %s", sysfs_path));
+                } else {
+                        long long int sync_pos, sync_total;
+
+                        if (sscanf (str_completed, "%lld / %lld", &sync_pos, &sync_total) != 2) {
+                                HAL_WARNING (("Malformed sync_completed '%s'", str_completed));
+                        } else {
+                                double sync_progress;
+                                sync_progress = ((double) sync_pos) / ((double) sync_total);
+                                hal_device_property_set_double (d, "storage.linux_raid.sync.progress", sync_progress);
+                        }
+                }
+
+                /* check again in two seconds */
+                g_timeout_add (2000, md_check_sync_timeout, g_strdup (sysfs_path));
+        }
+        
+        if (!hal_util_get_int_from_file (sysfs_path, "md/raid_disks", &num_components, 0)) {
+                HAL_WARNING (("Cannot get number of RAID components"));
+                goto error;
+        }
+        hal_device_property_set_int (d, "storage.linux_raid.num_components", num_components);
+        
+        /* add all components */
+        for (n = 0; n < num_components; n++) {
+                char *s;
+                char *link;
+                char *target;
+                HalDevice *slave_volume;
+                const char *slave_volume_stordev_udi;
+                HalDevice *slave_volume_stordev;
+
+                s = g_strdup_printf ("%s/md/rd%d", sysfs_path, n);
+                if (!g_file_test (s, G_FILE_TEST_IS_SYMLINK)) {
+                        g_free (s);
+                        break;
+                }
+                g_free (s);
+                
+                link = g_strdup_printf ("%s/md/rd%d/block", sysfs_path, n);
+                target = resolve_symlink (link);
+                if (target == NULL) {
+                        HAL_WARNING (("Cannot resolve %s", link));
+                        g_free (link);
+                        goto error;
+                }
+                HAL_INFO (("link->target: '%s' -> '%s'", link, target));
+                
+                slave_volume = hal_device_store_match_key_value_string (hald_get_gdl (), "linux.sysfs_path", target);
+                if (slave_volume == NULL) {
+                        HAL_WARNING (("No volume for sysfs path %s", target));
+                        g_free (target);
+                        g_free (link);
+                        goto error;
+                }
+                
+                
+                slave_volume_stordev_udi = hal_device_property_get_string (slave_volume, "block.storage_device");
+                if (slave_volume_stordev_udi == NULL) {
+                        HAL_WARNING (("No storage device for slave"));
+                        g_free (target);
+                        g_free (link);
+                        goto error;
+                }
+                slave_volume_stordev = hal_device_store_find (hald_get_gdl (), slave_volume_stordev_udi);
+                if (slave_volume_stordev == NULL) {
+                        HAL_WARNING (("No storage device for slave"));
+                        g_free (target);
+                        g_free (link);
+                        goto error;
+                }
+                
+                
+                hal_device_property_strlist_add (d, "storage.linux_raid.components", hal_device_get_udi (slave_volume));
+                
+                /* derive 
+                 *
+                 * - hotpluggability (is that a word?) 
+                 * - array uuid
+                 *
+                 * Hmm.. every raid member (PV) get the array UUID. That's
+                 * probably.. wrong. TODO: check with Kay.
+                 *
+                 * from component 0.
+                 */
+                if (n == 0) {
+                        const char *uuid;
+
+                        hal_device_property_set_bool (
+                                d, "storage.hotpluggable",
+                                hal_device_property_get_bool (slave_volume_stordev, "storage.hotpluggable"));
+
+                        
+                        uuid = hal_device_property_get_string (
+                                slave_volume, "volume.uuid");
+                        if (uuid != NULL) {
+                                hal_device_property_set_string (
+                                        d, "storage.serial", uuid);
+                        }
+                }
+                
+                g_free (target);
+                g_free (link);
+                
+        } /* for all components */
+
+        hal_device_property_set_int (d, "storage.linux_raid.num_components_active", n);
+        
+        /* TODO: add more state here */
+
+        ret = TRUE;
+
+error:
+        return ret;
+}
+
+
 void
 hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_file, gboolean is_partition,
 				  HalDevice *parent, void *end_token)
@@ -638,8 +845,12 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 	char *sysfs_path_real = NULL;
 	int floppy_num;
 	gboolean is_device_mapper;
+        gboolean is_md_device;
+        int md_number;
 
 	is_device_mapper = FALSE;
+        is_fakevolume = FALSE;
+        is_md_device = FALSE;
 
 	HAL_INFO (("block_add: sysfs_path=%s dev=%s is_part=%d, parent=0x%08x", 
 		   sysfs_path, device_file, is_partition, parent));
@@ -650,11 +861,18 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 	}
 
 	if (strcmp (hal_util_get_last_element (sysfs_path), "fakevolume") == 0) {
+		HAL_INFO (("Handling %s as fakevolume - sysfs_path_real=%s", device_file, sysfs_path_real));
 		is_fakevolume = TRUE;
 		sysfs_path_real = hal_util_get_parent_path (sysfs_path);
-		HAL_INFO (("Handling %s as fakevolume - sysfs_path_real=%s", device_file, sysfs_path_real));
+        } else if (sscanf (hal_util_get_last_element (sysfs_path), "md%d", &md_number) == 1) {
+		HAL_INFO (("Handling %s as MD device", device_file));
+                is_md_device = TRUE;
+		sysfs_path_real = g_strdup (sysfs_path);
+                /* set parent to root computer device object */
+                parent = hal_device_store_find (hald_get_gdl (), "/org/freedesktop/Hal/devices/computer");
+                if (parent == NULL)
+                        d = hal_device_store_find (hald_get_tdl (), "/org/freedesktop/Hal/devices/computer");
 	} else {
-		is_fakevolume = FALSE;
 		sysfs_path_real = g_strdup (sysfs_path);
 	}
 	
@@ -670,13 +888,13 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 	d = hal_device_new ();
 
 	/* OK, no parent... it might a device-mapper device => check slaves/ subdir in sysfs */
-	if (parent == NULL && !is_partition && !is_fakevolume) {
+	if (parent == NULL && !is_partition && !is_fakevolume && !hotplug_event->reposted) {
 		DIR * dir;
 		struct dirent *dp;
 		char path[HAL_PATH_MAX];
 
 		g_snprintf (path, HAL_PATH_MAX, "%s/slaves", sysfs_path);
-		HAL_INFO (("Looking in %s", path));
+		HAL_INFO (("Looking in %s for Device Mapper", path));
 
 		if ((dir = opendir (path)) == NULL) {
 			HAL_WARNING (("Unable to open %s: %s", path, strerror(errno)));
@@ -725,7 +943,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 			closedir(dir);
 			HAL_INFO (("Done looking in %s", path));
 		}
-		
+
 	}
 
 	if (parent == NULL) {
@@ -856,6 +1074,53 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 				}
 			}
 
+                        if (strcmp (udi_it, "/org/freedesktop/Hal/devices/computer") == 0) {
+					physdev = d_it;
+					physdev_udi = udi_it;
+                                        if (is_md_device) {
+                                                char *level;
+                                                char *model_name;
+
+                                                hal_device_property_set_string (d, "storage.bus", "linux_raid");
+
+                                                level = hal_util_get_string_from_file (sysfs_path_real, "md/level");
+                                                if (level == NULL)
+                                                        goto error;
+                                                hal_device_property_set_string (d, "storage.linux_raid.level", level);
+
+                                                hal_device_property_set_string (d, "storage.linux_raid.sysfs_path", sysfs_path_real);
+
+                                                hal_device_property_set_string (d, "storage.vendor", "Linux");
+                                                if (strcmp (level, "linear") == 0) {
+                                                        model_name = g_strdup ("Software RAID (Linear)");
+                                                } else if (strcmp (level, "raid0") == 0) {
+                                                        model_name = g_strdup ("Software RAID-0 (Stripe)");
+                                                } else if (strcmp (level, "raid1") == 0) {
+                                                        model_name = g_strdup ("Software RAID-1 (Mirror)");
+                                                } else if (strcmp (level, "raid5") == 0) {
+                                                        model_name = g_strdup ("Software RAID-5");
+                                                } else {
+                                                        model_name = g_strdup_printf ("Software RAID (%s)", level);
+                                                }
+                                                hal_device_property_set_string (d, "storage.model", model_name);
+                                                g_free (model_name);
+
+                                                hal_util_set_string_from_file (
+                                                        d, "storage.firmware_version", 
+                                                        sysfs_path_real, "md/metadata_version");
+                                                
+                                                hal_device_add_capability (d, "storage.linux_raid");
+
+                                                if (!refresh_md_state (d))
+                                                        goto error;
+
+                                                is_hotpluggable = hal_device_property_get_bool (
+                                                        d, "storage.hotpluggable");
+
+                                        }
+                                        break;
+                        }
+
 			/* Check info.subsystem */
 			if ((bus = hal_device_property_get_string (d_it, "info.subsystem")) != NULL) {
 				if (strcmp (bus, "scsi") == 0) {
@@ -918,9 +1183,9 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		hal_device_property_set_string (d, "storage.originating_device", physdev_udi);
 		hal_device_property_set_string (d, "storage.physical_device", physdev_udi);
 
-		if (!hal_util_get_int_from_file (sysfs_path, "removable", (gint *) &is_removable, 10)) {
+		if (!hal_util_get_int_from_file (sysfs_path_real, "removable", (gint *) &is_removable, 10)) {
 			HAL_WARNING (("Cannot get 'removable' file"));
-			goto error;
+                        is_removable = FALSE;
 		}
 
 		hal_device_property_set_bool (d, "storage.removable.media_available", FALSE);
@@ -928,7 +1193,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		/* set storage.size only if we have fixed media */
 		if (!is_removable) {
 			guint64 num_blocks;
-			if (hal_util_get_uint64_from_file (sysfs_path, "size", &num_blocks, 0)) {
+			if (hal_util_get_uint64_from_file (sysfs_path_real, "size", &num_blocks, 0)) {
 				/* TODO: sane to assume this is always 512 for non-removable? 
 				 * I think genhd.c guarantees this... */
 				hal_device_property_set_uint64 (d, "storage.size", num_blocks * 512);
@@ -937,6 +1202,17 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 			hal_device_property_set_uint64 (d, "storage.size", 0);
 		}
 
+                if (is_removable) {
+                        int sysfs_capability;
+                        gboolean support_an;
+
+                        support_an = 
+                                hal_util_get_int_from_file (sysfs_path, "capability", &sysfs_capability, 16) &&
+                                (sysfs_capability&4) != 0;
+                        
+                        hal_device_property_set_bool (d, "storage.removable.support_async_notification", support_an);
+                }
+
 		/* by default, do checks for media if, and only if, the removable file is set to 1
 		 *
 		 * Problematic buses, like IDE, may override this.
@@ -955,6 +1231,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 			char buf[256];
 			gchar *media;
 			gchar *model;
+			struct stat st;
 
 			/* Be conservative and don't poll IDE drives at all (except CD-ROM's, see below) */
 			hal_device_property_set_bool (d, "storage.media_check_enabled", FALSE);
@@ -963,7 +1240,13 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 			 *
 			 * "disk", "cdrom", "tape", "floppy", "UNKNOWN"
 			 */
-			snprintf (buf, sizeof (buf), "%s/ide/%s", get_hal_proc_path (), hal_util_get_last_element (sysfs_path));
+			snprintf (buf, sizeof (buf), "/proc/ide/%s", hal_util_get_last_element (sysfs_path_real));
+			if (stat(buf, &st)) {
+				/*
+				 * /proc/ide does not exist; try with sysfs
+				 */
+				snprintf (buf, sizeof (buf), "%s/%s", sysfs_path_real, "device");
+			}
 			if ((media = hal_util_get_string_from_file (buf, "media")) != NULL) {
 				if (strcmp (media, "disk") == 0 ||
 				    strcmp (media, "cdrom") == 0 ||
@@ -1152,10 +1435,12 @@ error:
 	if (d != NULL)
 		g_object_unref (d);
 out:
-	hotplug_event_end (end_token);
+        hotplug_event_end (end_token);
 	g_free (sysfs_path_real);
+        return;
 }
 
+
 static void
 force_unmount_cb (HalDevice *d, guint32 exit_type, 
 		  gint return_code, gchar **error,
@@ -1272,6 +1557,20 @@ out:
 	;
 }
 
+void 
+hotplug_event_refresh_blockdev (gchar *sysfs_path, HalDevice *d, void *end_token)
+{
+	HAL_INFO (("block_change: sysfs_path=%s", sysfs_path));
+
+        if (hal_device_property_get_bool (d, "storage.removable.support_async_notification")) {
+                blockdev_rescan_device (d);
+        }
+
+	/* done with change event */
+	hotplug_event_end (end_token);
+}
+
+
 static void 
 block_rescan_storage_done (HalDevice *d, guint32 exit_type, 
                            gint return_code, gchar **error,
@@ -1325,7 +1624,7 @@ blockdev_rescan_device (HalDevice *d)
 
 	ret = FALSE;
 
-	HAL_INFO (("Entering, udi=%s", hal_device_get_udi (d)));
+	HAL_INFO (("blockdev_rescan_device: udi=%s", hal_device_get_udi (d)));
 
 	/* This only makes sense on storage devices */
 	if (hal_device_property_get_bool (d, "block.is_volume")) {
@@ -1403,3 +1702,218 @@ blockdev_generate_remove_hotplug_event (HalDevice *d)
 
 	return hotplug_event;
 }
+
+static GSList *md_devs = NULL;
+
+static char *
+udev_get_device_file_for_sysfs_path (const char *sysfs_path)
+{
+        char *ret;
+        char *u_stdout;
+        int u_exit_status;
+        const char *argv[] = {"/usr/bin/udevinfo", "--root", "--query", "name", "--path", NULL, NULL};
+        GError *g_error;
+
+        ret = NULL;
+        argv[5] = sysfs_path;
+
+        g_error = NULL;
+
+        if (!g_spawn_sync("/", 
+                          (char **) argv, 
+                          NULL,           /* envp */
+                          0,              /* flags */
+                          NULL,           /* child_setup */
+                          NULL,           /* user_data */
+                          &u_stdout,
+                          NULL,           /* stderr */
+                          &u_exit_status,
+                          &g_error)) {
+                HAL_ERROR (("Error spawning udevinfo: %s", g_error->message));
+                g_error_free (g_error);
+                goto out;
+        }
+
+        if (u_exit_status != 0) {
+                HAL_ERROR (("udevinfo returned exit code %d", u_exit_status));
+                g_free (u_stdout);
+                goto out;
+        }
+
+        ret = u_stdout;
+        g_strchomp (ret);
+        HAL_INFO (("Got '%s'", ret));
+
+out:
+        return ret;
+}
+
+
+void 
+blockdev_process_mdstat (void)
+{
+	HotplugEvent *hotplug_event;
+        GIOChannel *channel;
+        GSList *read_md_devs;
+        GSList *i;
+        GSList *j;
+        GSList *k;
+
+        channel = get_mdstat_channel ();
+        if (channel == NULL)
+                goto error;
+
+        if (g_io_channel_seek (channel, 0, G_SEEK_SET) != G_IO_ERROR_NONE) {
+                HAL_ERROR (("Cannot seek in /proc/mdstat"));
+                goto error;
+        }
+
+        read_md_devs = NULL;
+        while (TRUE) {
+                int num;
+                char *line;
+
+                if (g_io_channel_read_line (channel, &line, NULL, NULL, NULL) != G_IO_STATUS_NORMAL)
+                        break;
+
+                if (sscanf (line, "md%d : ", &num) == 1) {
+                        char *sysfs_path;
+                        sysfs_path = g_strdup_printf ("/sys/block/md%d", num);
+                        read_md_devs = g_slist_prepend (read_md_devs, sysfs_path);
+                }
+
+                g_free (line);
+        }
+
+        /* now compute the delta */
+
+        /* add devices */
+        for (i = read_md_devs; i != NULL; i = i->next) {
+                gboolean should_add = TRUE;
+
+                for (j = md_devs; j != NULL && should_add; j = j->next) {
+                        if (strcmp (i->data, j->data) == 0) {
+                                should_add = FALSE;
+                        }
+                }
+
+                if (should_add) {
+                        char *sysfs_path = i->data;
+                        char *device_file;
+                        int num_tries;
+
+                        num_tries = 0;
+                retry_add:
+                        device_file = udev_get_device_file_for_sysfs_path (sysfs_path);
+                        if (device_file == NULL) {
+                                if (num_tries <= 6) {
+                                        int num_ms;
+                                        num_ms = 10 * (1<<num_tries);
+                                        HAL_INFO (("spinning %d ms waiting for device file for sysfs path %s", 
+                                                   num_ms, sysfs_path));
+                                        usleep (1000 * num_ms);
+                                        num_tries++;
+                                        goto retry_add;
+                                } else {
+                                        HAL_ERROR (("Cannot get device file for sysfs path %s", sysfs_path));
+                                }
+                        } else {
+                                HAL_INFO (("Adding md device at '%s' ('%s')", sysfs_path, device_file));
+
+                                hotplug_event = g_new0 (HotplugEvent, 1);
+                                hotplug_event->action = HOTPLUG_ACTION_ADD;
+                                hotplug_event->type = HOTPLUG_EVENT_SYSFS_BLOCK;
+                                g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
+                                g_strlcpy (hotplug_event->sysfs.sysfs_path, sysfs_path, sizeof (hotplug_event->sysfs.sysfs_path));
+                                g_strlcpy (hotplug_event->sysfs.device_file, device_file, sizeof (hotplug_event->sysfs.device_file));
+                                hotplug_event->sysfs.net_ifindex = -1;
+                                hotplug_event_enqueue (hotplug_event);
+                                
+                                md_devs = g_slist_prepend (md_devs, g_strdup (sysfs_path));
+
+                                g_free (device_file);
+                        }
+
+                }
+        }
+
+        /* remove devices */
+        for (i = md_devs; i != NULL; i = k) {
+                gboolean should_remove = TRUE;
+
+                k = i->next;
+
+                for (j = read_md_devs; j != NULL && should_remove; j = j->next) {
+                        if (strcmp (i->data, j->data) == 0) {
+                                should_remove = FALSE;
+                        }
+                }
+
+                if (should_remove) {
+                        char *sysfs_path = i->data;
+                        char *device_file;
+                        int num_tries;
+                        
+                        num_tries = 0;
+                retry_rem:
+                        device_file = udev_get_device_file_for_sysfs_path (sysfs_path);
+                        if (device_file == NULL) {
+                                if (num_tries <= 6) {
+                                        int num_ms;
+                                        num_ms = 10 * (1<<num_tries);
+                                        HAL_INFO (("spinning %d ms waiting for device file for sysfs path %s", 
+                                                   num_ms, sysfs_path));
+                                        usleep (1000 * num_ms);
+                                        num_tries++;
+                                        goto retry_rem;
+                                } else {
+                                        HAL_ERROR (("Cannot get device file for sysfs path %s", sysfs_path));
+                                }
+                        } else {
+                                
+                                HAL_INFO (("Removing md device at '%s' ('%s')", sysfs_path, device_file));
+
+                                hotplug_event = g_new0 (HotplugEvent, 1);
+                                hotplug_event->action = HOTPLUG_ACTION_REMOVE;
+                                hotplug_event->type = HOTPLUG_EVENT_SYSFS_BLOCK;
+                                g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
+                                g_strlcpy (hotplug_event->sysfs.sysfs_path, sysfs_path, sizeof (hotplug_event->sysfs.sysfs_path));
+                                g_strlcpy (hotplug_event->sysfs.device_file, device_file, sizeof (hotplug_event->sysfs.device_file));
+                                hotplug_event->sysfs.net_ifindex = -1;
+                                hotplug_event_enqueue (hotplug_event);
+                                
+                                md_devs = g_slist_remove_link (md_devs, i);
+                                g_free (i->data);
+                                g_slist_free (i);
+
+                                g_free (device_file);
+                        }
+
+                }
+        }
+
+        g_slist_foreach (read_md_devs, (GFunc) g_free, NULL);
+        g_slist_free (read_md_devs);
+
+        /* finally, refresh all md devices */
+        for (i = md_devs; i != NULL; i = i->next) {
+                char *sysfs_path = i->data;
+                HalDevice *d;
+
+                d = hal_device_store_match_key_value_string (hald_get_gdl (), 
+                                                             "storage.linux_raid.sysfs_path", 
+                                                             sysfs_path);
+                if (d == NULL)
+                        d = hal_device_store_match_key_value_string (hald_get_tdl (), 
+                                                                     "storage.linux_raid.sysfs_path", 
+                                                                     sysfs_path);
+                if (d != NULL)
+                        refresh_md_state (d);
+        }
+        
+
+        hotplug_event_process_queue ();
+
+error:
+        ;
+}
