@@ -42,6 +42,7 @@
 #include "osspec_linux.h"
 #include "hotplug.h"
 #include "coldplug.h"
+#include "blockdev.h"
 
 #define DMPREFIX "dm-"
 
@@ -132,7 +133,7 @@ hal_util_init_sysfs_to_udev_map (void)
 		/* new device */
 		if (strncmp(line, "P: ", 3) == 0) {
 			hotplug_event = g_new0 (HotplugEvent, 1);
-			g_strlcpy (hotplug_event->sysfs.sysfs_path, get_hal_sysfs_path (), sizeof(hotplug_event->sysfs.sysfs_path));
+			g_strlcpy (hotplug_event->sysfs.sysfs_path, "/sys", sizeof(hotplug_event->sysfs.sysfs_path));
 			g_strlcat (hotplug_event->sysfs.sysfs_path, &line[3], sizeof(hotplug_event->sysfs.sysfs_path));
 			continue;
 		}
@@ -333,14 +334,42 @@ error:
 	return -1;
 }
 
+static void
+scan_single_bus (const char *bus_name)
+{
+        char dirname[HAL_PATH_MAX];
+        DIR *dir2;
+        struct dirent *dent2;
+        
+        g_strlcpy(dirname, "/sys/bus/", sizeof(dirname));
+        g_strlcat(dirname, bus_name, sizeof(dirname));
+        g_strlcat(dirname, "/devices", sizeof(dirname));
+        
+        /* look for devices */
+        dir2 = opendir(dirname);
+        if (dir2 != NULL) {
+                for (dent2 = readdir(dir2); dent2 != NULL; dent2 = readdir(dir2)) {
+                        char dirname2[HAL_PATH_MAX];
+                        
+                        if (dent2->d_name[0] == '.')
+                                continue;
+                        
+                        g_strlcpy(dirname2, dirname, sizeof(dirname2));
+                        g_strlcat(dirname2, "/", sizeof(dirname2));
+                        g_strlcat(dirname2, dent2->d_name, sizeof(dirname2));
+                        device_list_insert(dirname2, bus_name, HOTPLUG_EVENT_SYSFS_DEVICE);
+                }
+                closedir(dir2);
+        }
+}
+
 static void scan_subsystem(const char *subsys)
 {
 	char base[HAL_PATH_MAX];
 	DIR *dir;
 	struct dirent *dent;
 
-	g_strlcpy(base, get_hal_sysfs_path (), sizeof(base));
-	g_strlcat(base, "/", sizeof(base));
+	g_strlcpy(base, "/sys/", sizeof(base));
 	g_strlcat(base, subsys, sizeof(base));
 
 	dir = opendir(base);
@@ -381,14 +410,10 @@ static void scan_subsystem(const char *subsys)
 
 static void scan_block(void)
 {
-	char base[HAL_PATH_MAX];
 	DIR *dir;
 	struct dirent *dent;
 
-	g_strlcpy(base, get_hal_sysfs_path (), sizeof(base));
-	g_strlcat(base, "/block", sizeof(base));
-
-	dir = opendir(base);
+	dir = opendir("/sys/block");
 	if (dir != NULL) {
 		for (dent = readdir(dir); dent != NULL; dent = readdir(dir)) {
 			char dirname[HAL_PATH_MAX];
@@ -398,8 +423,13 @@ static void scan_block(void)
 			if (dent->d_name[0] == '.')
 				continue;
 
-			g_strlcpy(dirname, base, sizeof(dirname));
-			g_strlcat(dirname, "/", sizeof(dirname));
+                        /* md devices are handled via looking at /proc/mdstat */
+                        if (g_str_has_prefix (dent->d_name, "md")) {
+                                HAL_INFO (("skipping md event for %s", dent->d_name));
+                                continue;
+                        }
+
+			g_strlcpy(dirname, "/sys/block/", sizeof(dirname));
 			g_strlcat(dirname, dent->d_name, sizeof(dirname));
 			if (device_list_insert(dirname, "block", HOTPLUG_EVENT_SYSFS_BLOCK) != 0)
 				continue;
@@ -429,14 +459,10 @@ static void scan_block(void)
 
 static void scan_class(void)
 {
-	char base[HAL_PATH_MAX];
 	DIR *dir;
 	struct dirent *dent;
 
-	g_strlcpy(base, get_hal_sysfs_path (), sizeof(base));
-	g_strlcat(base, "/class", sizeof(base));
-
-	dir = opendir(base);
+	dir = opendir("/sys/class");
 	if (dir != NULL) {
 		for (dent = readdir(dir); dent != NULL; dent = readdir(dir)) {
 			char dirname[HAL_PATH_MAX];
@@ -446,8 +472,7 @@ static void scan_class(void)
 			if (dent->d_name[0] == '.')
 				continue;
 
-			g_strlcpy(dirname, base, sizeof(dirname));
-			g_strlcat(dirname, "/", sizeof(dirname));
+			g_strlcpy(dirname, "/sys/class/", sizeof(dirname));
 			g_strlcat(dirname, dent->d_name, sizeof(dirname));
 			dir2 = opendir(dirname);
 			if (dir2 != NULL) {
@@ -500,9 +525,9 @@ static int _device_order (const void *d1, const void *d2)
 	const struct sysfs_device *dev2 = d2;
 
 	/* device mapper needs to be the last events, to have the other block devs already around */
-	if (strstr(dev2->path, "/" DMPREFIX))
+	if (strstr (dev2->path, "/" DMPREFIX))
 		return -1;
-	if (strstr(dev1->path, "/" DMPREFIX))
+	if (strstr (dev1->path, "/" DMPREFIX))
 		return 1;
 
 	return strcmp(dev1->path, dev2->path);
@@ -511,7 +536,6 @@ static int _device_order (const void *d1, const void *d2)
 gboolean
 coldplug_synthesize_events (void)
 {
-	char base[HAL_PATH_MAX];
 	struct stat statbuf;
 
 	if (hal_util_init_sysfs_to_udev_map () == FALSE) {
@@ -520,9 +544,7 @@ coldplug_synthesize_events (void)
 	}
 
 	/* if we have /sys/subsystem, forget all the old stuff */
-	g_strlcpy(base, get_hal_sysfs_path (), sizeof(base));
-	g_strlcat(base, "/subsystem", sizeof(base));
-	if (stat(base, &statbuf) == 0) {
+	if (stat("/sys/subsystem", &statbuf) == 0) {
 		scan_subsystem ("subsystem");
 		device_list = g_slist_sort (device_list, _device_order);
 		queue_events ();
@@ -532,17 +554,19 @@ coldplug_synthesize_events (void)
 		queue_events ();
 
 		scan_class ();
+                scan_single_bus ("bluetooth");
 		device_list = g_slist_sort (device_list, _device_order);
 		queue_events ();
 
 		/* scan /sys/block, if it isn't already a class */
-		g_strlcpy(base, get_hal_sysfs_path (), sizeof(base));
-		g_strlcat(base, "/class/block", sizeof(base));
-		if (stat(base, &statbuf) != 0) {
+		if (stat("/sys/class/block", &statbuf) != 0) {
 			scan_block ();
 			device_list = g_slist_sort (device_list, _device_order);
 			queue_events ();
 		}
+
+                /* add events from reading /proc/mdstat */
+                blockdev_process_mdstat ();
 	}
 
 	g_hash_table_destroy (sysfs_to_udev_map);
