@@ -88,13 +88,21 @@ hotplug_event_begin_sysfs (HotplugEvent *hotplug_event)
 	d = hal_device_store_match_key_value_string (hald_get_gdl (),
 						     "linux.sysfs_path",
 						     hotplug_event->sysfs.sysfs_path);
+	
+	if (d == NULL && hotplug_event->action == HOTPLUG_ACTION_MOVE) {
+		d = hal_device_store_match_key_value_string(hald_get_gdl (),
+							    "linux.sysfs_path",
+							    hotplug_event->sysfs.sysfs_path_old);
+	}
 
-	/* FIXME: we should reprobe the device instead of skipping the event */
+#if 0
+	/* we should refresh the device when we get "change" uevent */
 	if (d != NULL && hotplug_event->action == HOTPLUG_ACTION_ADD) {
-		HAL_ERROR (("devpath %s already present in the store, ignore event", hotplug_event->sysfs.sysfs_path));
-		hotplug_event_end ((void *) hotplug_event);
+		HAL_ERROR (("device %s already present in the store, so refreshing", hotplug_event->sysfs.sysfs_path));
+		hotplug_event_refresh_dev (hotplug_event->sysfs.subsystem, d, (void *) hotplug_event);
 		return;
 	}
+#endif
 
 	/* subsystem "block" are all block devices */
 	if (hotplug_event->type == HOTPLUG_EVENT_SYSFS)
@@ -142,7 +150,7 @@ hotplug_event_begin_sysfs (HotplugEvent *hotplug_event)
 		char sys_block_path[HAL_PATH_MAX];
 		gsize sys_block_path_len;
 
-		sys_block_path_len   = g_snprintf (sys_block_path, HAL_PATH_MAX, "%s/block", get_hal_sysfs_path ());
+		sys_block_path_len = g_snprintf (sys_block_path, HAL_PATH_MAX, "/sys/block");
 		if (strncmp (hotplug_event->sysfs.sysfs_path, sys_block_path, sys_block_path_len) == 0) {
 			HAL_INFO (("%s is a block device (devpath)", hotplug_event->sysfs.sysfs_path));
 			hotplug_event->type = HOTPLUG_EVENT_SYSFS_BLOCK;
@@ -151,7 +159,8 @@ hotplug_event_begin_sysfs (HotplugEvent *hotplug_event)
 	}
 
 	if (hotplug_event->type == HOTPLUG_EVENT_SYSFS_DEVICE) {
-		if (hotplug_event->action == HOTPLUG_ACTION_ADD) {
+		if (hotplug_event->action == HOTPLUG_ACTION_ADD ||
+		    (d == NULL && hotplug_event->action == HOTPLUG_ACTION_CHANGE)) {
 			HalDevice *parent;
 			gchar *parent_path;
 
@@ -168,9 +177,23 @@ hotplug_event_begin_sysfs (HotplugEvent *hotplug_event)
 			hotplug_event_begin_remove_dev (hotplug_event->sysfs.subsystem,
 							     hotplug_event->sysfs.sysfs_path,
 							     (void *) hotplug_event);
+		} else if (hotplug_event->action == HOTPLUG_ACTION_CHANGE && d != NULL) {
+			hotplug_event_refresh_dev (hotplug_event->sysfs.subsystem,
+                                                   hotplug_event->sysfs.sysfs_path,
+                                                   d,
+                                                   (void *) hotplug_event);
+		} else if (hotplug_event->action == HOTPLUG_ACTION_MOVE && d != NULL) {
+			hal_device_property_set_string (d, "linux.sysfs_path", hotplug_event->sysfs.sysfs_path);
+			hotplug_event_refresh_dev (hotplug_event->sysfs.subsystem,
+                                                   hotplug_event->sysfs.sysfs_path,
+                                                   d,
+                                                   (void *) hotplug_event);
+		} else {
+			hotplug_event_end ((void *) hotplug_event);
 		}
 	} else if (hotplug_event->type == HOTPLUG_EVENT_SYSFS_BLOCK) {
-		if (hotplug_event->action == HOTPLUG_ACTION_ADD) {
+		if (hotplug_event->action == HOTPLUG_ACTION_ADD ||
+		    (d == NULL && hotplug_event->action == HOTPLUG_ACTION_CHANGE)) {
 			HalDevice *parent;
 			int range;
 			gboolean is_partition;
@@ -184,10 +207,11 @@ hotplug_event_begin_sysfs (HotplugEvent *hotplug_event)
 			 * from within HAL for partitions on the main block device
 			 */
 			if ((strstr (hotplug_event->sysfs.sysfs_path, "/fakevolume") != NULL) ||
-			    hal_util_get_int_from_file (hotplug_event->sysfs.sysfs_path, "range", &range, 0))
+                            hal_util_get_int_from_file (hotplug_event->sysfs.sysfs_path, "range", &range, 0)) {
 				is_partition = FALSE;
-			else
+			} else {
 				is_partition = TRUE;
+                        }
 
 			hal_util_find_known_parent (hotplug_event->sysfs.sysfs_path, &parent, NULL);
 			hotplug_event_begin_add_blockdev (hotplug_event->sysfs.sysfs_path,
@@ -198,6 +222,12 @@ hotplug_event_begin_sysfs (HotplugEvent *hotplug_event)
 		} else if (hotplug_event->action == HOTPLUG_ACTION_REMOVE) {
 			hotplug_event_begin_remove_blockdev (hotplug_event->sysfs.sysfs_path,
 							     (void *) hotplug_event);
+		} else if (hotplug_event->action == HOTPLUG_ACTION_CHANGE && d != NULL) {
+			hotplug_event_refresh_blockdev (hotplug_event->sysfs.sysfs_path,
+                                                        d,
+                                                        (void *) hotplug_event);
+		} else {
+			hotplug_event_end ((void *) hotplug_event);
 		}
 	} else {
 		/* just ignore this hotplug event */
