@@ -44,12 +44,6 @@
 #include <sys/utsname.h>
 #include <unistd.h>
 
-#ifdef HAVE_SYS_INOTIFY_H
-#include <sys/inotify.h>
-#else
-#include "inotify_local.h"
-#endif
-
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 
@@ -71,20 +65,7 @@
 
 #include "osspec_linux.h"
 
-static char *hal_sysfs_path;
-static char *hal_proc_path;
-
-const gchar *
-get_hal_sysfs_path (void)
-{
-	return hal_sysfs_path;
-}
-
-const gchar *
-get_hal_proc_path (void)
-{
-	return hal_proc_path;
-}
+static gboolean hald_done_synthesizing_coldplug = FALSE;
 
 static gboolean
 hald_udev_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
@@ -155,10 +136,27 @@ hald_udev_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
 
 		if (strncmp(key, "ACTION=", 7) == 0)
 			action = &key[7];
-		else if (strncmp(key, "DEVPATH=", 8) == 0)
+		else if (strncmp(key, "DEVPATH=", 8) == 0) {
+
+                        /* md devices are handled via looking at /proc/mdstat */
+                        if (g_str_has_prefix (key + 8, "/block/md")) {
+                                HAL_INFO (("skipping md event for %s", key + 8));
+                                goto invalid;
+                        }
+
 			g_snprintf (hotplug_event->sysfs.sysfs_path, sizeof (hotplug_event->sysfs.sysfs_path),
-				    "%s%s", hal_sysfs_path, &key[8]);
-		else if (strncmp(key, "SUBSYSTEM=", 10) == 0)
+				    "/sys%s", &key[8]);
+		} else if (strncmp(key, "DEVPATH_OLD=", 12) == 0) {
+
+                        /* md devices are handled via looking at /proc/mdstat */
+                        if (g_str_has_prefix (key + 12, "/block/md")) {
+                                HAL_INFO (("skipping md event for %s", key + 8));
+                                goto invalid;
+                        }
+
+			g_snprintf (hotplug_event->sysfs.sysfs_path_old, sizeof (hotplug_event->sysfs.sysfs_path_old),
+				    "/sys%s", &key[12]);
+		} else if (strncmp(key, "SUBSYSTEM=", 10) == 0)
 			g_strlcpy (hotplug_event->sysfs.subsystem, &key[10], sizeof (hotplug_event->sysfs.subsystem));
 		else if (strncmp(key, "DEVNAME=", 8) == 0)
 			g_strlcpy (hotplug_event->sysfs.device_file, &key[8], sizeof (hotplug_event->sysfs.device_file));
@@ -231,13 +229,27 @@ hald_udev_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
 		   hotplug_event->sysfs.seqnum, action, hotplug_event->sysfs.subsystem, hotplug_event->sysfs.sysfs_path,
 		   hotplug_event->sysfs.device_file, hotplug_event->sysfs.net_ifindex));
 
-	if (strcmp (action, "add") == 0 || strcmp (action, "change") == 0) {
+	if (strcmp (action, "add") == 0) {
 		hotplug_event->action = HOTPLUG_ACTION_ADD;
 		hotplug_event_enqueue (hotplug_event);
 		hotplug_event_process_queue ();
 		goto out;
 	}
 
+	if (strcmp (action, "change") == 0) {
+		hotplug_event->action = HOTPLUG_ACTION_CHANGE;
+		hotplug_event_enqueue (hotplug_event);
+		hotplug_event_process_queue ();
+		goto out;
+	}
+
+	if (strcmp (action, "move") == 0) {
+		hotplug_event->action = HOTPLUG_ACTION_MOVE;
+		hotplug_event_enqueue (hotplug_event);
+		hotplug_event_process_queue ();
+		goto out;
+	}
+
 	if (strcmp (action, "remove") == 0) {
 		hotplug_event->action = HOTPLUG_ACTION_REMOVE;
 		hotplug_event_enqueue (hotplug_event);
@@ -253,8 +265,7 @@ out:
 }
 
 static gboolean
-mount_tree_changed_event (GIOChannel *channel, GIOCondition cond,
-		    gpointer user_data)
+mount_tree_changed_event (GIOChannel *channel, GIOCondition cond, gpointer user_data)
 {
 	if (cond & ~G_IO_ERR)
 		return TRUE;
@@ -265,134 +276,67 @@ mount_tree_changed_event (GIOChannel *channel, GIOCondition cond,
 	return TRUE;
 }
 
-void
-osspec_privileged_init (void)
-{
-}
-
 static gboolean
-inotify_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
+mdstat_changed_event (GIOChannel *channel, GIOCondition cond, gpointer user_data)
 {
-	int fd;
-	char buf[256];
+	if (cond & ~G_IO_PRI)
+		return TRUE;
 
-	fd = g_io_channel_unix_get_fd (source);
-again:
-	if (read (fd, buf, sizeof (buf)) < 0) {
-		if (errno == EINTR) {
-			goto again;
-		} else {
-			HAL_ERROR (("read: %s", strerror (errno)));
-		}
-	}
+	HAL_INFO (("/proc/mdstat changed"));
 
-	/* TODO: OK, so the right thing to do would be to add watches
-	 * for directories that appear 
-	 */
+        blockdev_process_mdstat ();
 
-	osspec_fdi_cache_invalid ();
 	return TRUE;
 }
 
-static gboolean
-add_fdi_watch (int inotify_fd, const char *fdi_dir)
+static HalFileMonitor *file_monitor = NULL;
+
+HalFileMonitor *
+osspec_get_file_monitor (void)
 {
-	int wd;
-	gboolean ret;
-	GDir *dir;
-	GError *error = NULL;
-	const char *name;
-
-	ret = FALSE;
-
-	wd = inotify_add_watch (inotify_fd, fdi_dir, IN_MODIFY | IN_CREATE | IN_DELETE);
-	if (wd < 0) {
-		HAL_ERROR (("Unable to inotify_add_watch() for '%s': %s", fdi_dir, strerror (errno)));
-		goto out;
-	}
-
-	/* watch sub dirs */
-	dir = g_dir_open (fdi_dir, 0, &error);
-	if (dir == NULL)
-		goto out;
-	while ((name = g_dir_read_name (dir)) != NULL) {
-		char *sub_dir;
-
-		sub_dir = g_strdup_printf ("%s/%s", fdi_dir, name);
-		if (g_file_test (sub_dir, G_FILE_TEST_IS_DIR)) {
-			add_fdi_watch (inotify_fd, sub_dir);
-		}
-		g_free (sub_dir);
-	}
-	g_dir_close (dir);
-
-	ret = TRUE;
-out:
-	return ret;
+        return file_monitor;
 }
 
-static void
-watch_fdi_files (void)
+static GIOChannel *mdstat_channel = NULL;
+
+GIOChannel *get_mdstat_channel (void)
 {
-	int inotify_fd;
-	char *hal_fdi_source_preprobe;
-	char *hal_fdi_source_information;
-	char *hal_fdi_source_policy;
-	GIOChannel *inotify_channel;
-
-	hal_fdi_source_preprobe = getenv ("HAL_FDI_SOURCE_PREPROBE");
-	hal_fdi_source_information = getenv ("HAL_FDI_SOURCE_INFORMATION");
-	hal_fdi_source_policy = getenv ("HAL_FDI_SOURCE_POLICY");
-
-	inotify_fd = inotify_init ();
-	if (inotify_fd < 0) {
-		DIE (("Unable to initialize inotify: %s ", strerror (errno)));
-		goto out;
-	}
-	inotify_channel = g_io_channel_unix_new (inotify_fd);
-	g_io_add_watch (inotify_channel, G_IO_IN, inotify_data, NULL);
-	g_io_channel_unref (inotify_channel);
+        return mdstat_channel;
+}
 
 
-	if (hal_fdi_source_preprobe != NULL) {
-		if (!add_fdi_watch (inotify_fd, hal_fdi_source_preprobe))
-			goto out;
+void
+osspec_privileged_init (void)
+{
+	GError *err = NULL;
+
+        file_monitor = hal_file_monitor_new ();
+        if (file_monitor == NULL) {
+                DIE (("Cannot initialize file monitor"));
+        }
+
+	/* watch /proc/mdstat for md changes
+	 * kernel 2.6.19 throws a POLLPRI event for every change
+	 */
+	mdstat_channel = g_io_channel_new_file ("/proc/mdstat", "r", &err);
+	if (mdstat_channel != NULL) {
+		g_io_add_watch (mdstat_channel, G_IO_PRI, mdstat_changed_event, NULL);
 	} else {
-		if (!add_fdi_watch (inotify_fd, PACKAGE_DATA_DIR "/hal/fdi/preprobe"))
-			goto out;
-		if (!add_fdi_watch (inotify_fd, PACKAGE_SYSCONF_DIR "/hal/fdi/preprobe"))
-			goto out;
-	}
+		if (err != NULL)
+			HAL_WARNING (("Unable to open /proc/mdstat: %s", err->message));
 
-	if (hal_fdi_source_information != NULL) {
-		if (!add_fdi_watch (inotify_fd, hal_fdi_source_information))
-			goto out;
-	} else {
-		if (!add_fdi_watch (inotify_fd, PACKAGE_DATA_DIR "/hal/fdi/information"))
-			goto out;
-		if (!add_fdi_watch (inotify_fd, PACKAGE_SYSCONF_DIR "/hal/fdi/information"))
-			goto out;
+		/* if its not a reasonable error, abort */
+		if (!g_error_matches (err, G_FILE_ERROR, G_FILE_ERROR_NOENT))
+			DIE (("Unable to read /proc/mdstat"));
 	}
-
-	if (hal_fdi_source_policy != NULL) {
-		if (!add_fdi_watch (inotify_fd, hal_fdi_source_policy))
-			goto out;
-	} else {
-		if (!add_fdi_watch (inotify_fd, PACKAGE_DATA_DIR "/hal/fdi/policy"))
-			goto out;
-		if (!add_fdi_watch (inotify_fd, PACKAGE_SYSCONF_DIR "/hal/fdi/policy"))
-			goto out;
-	}
-
-	return;
-out:
-	DIE (("Error watching fdi files"));
+	
+	if (err != NULL)
+		g_error_free (err);
 }
 
 void
 osspec_init (void)
 {
-	gchar path[HAL_PATH_MAX];
 	int udev_socket;
 	struct sockaddr_un saddr;
 	socklen_t addrlen;
@@ -425,23 +369,10 @@ osspec_init (void)
 	g_io_add_watch (udev_channel, G_IO_IN, hald_udev_data, NULL);
 	g_io_channel_unref (udev_channel);
 
-	/*
-	 * set mount points for /proc and /sys, possibly overridden for testing
-	 */
-	hal_sysfs_path = getenv ("SYSFS_PATH");
-	if (hal_sysfs_path == NULL)
-		hal_sysfs_path = "/sys";
-
-	hal_proc_path = getenv ("PROC_PATH");
-	if (hal_proc_path == NULL)
-		hal_proc_path = "/proc";
-
-	/*
-	 * watch /proc/mounts for mount tree changes
+	/* watch /proc/mounts for mount tree changes
 	 * kernel 2.6.15 vfs throws a POLLERR event for every change
 	 */
-	g_snprintf (path, sizeof (path), "%s/mounts", get_hal_proc_path ());
-	mounts_channel = g_io_channel_new_file (path, "r", NULL);
+	mounts_channel = g_io_channel_new_file ("/proc/mounts", "r", NULL);
 	if (mounts_channel == NULL)
 		DIE (("Unable to read /proc/mounts"));
 	g_io_add_watch (mounts_channel, G_IO_ERR, mount_tree_changed_event, NULL);
@@ -452,9 +383,11 @@ osspec_init (void)
 	ids_init ();
 
 	/* watch fdi directories */
-	watch_fdi_files ();	
+	/* TODO: temporarily disabled... 
+           watch_fdi_files (); */
 }
 
+
 static void 
 computer_callouts_add_done (HalDevice *d, gpointer userdata1, gpointer userdata2)
 {
@@ -468,18 +401,48 @@ computer_callouts_add_done (HalDevice *d, gpointer userdata1, gpointer userdata2
 void
 hotplug_queue_now_empty (void)
 {
-	if (hald_is_initialising)
+	if (hald_is_initialising && hald_done_synthesizing_coldplug) {
 		osspec_probe_done ();
+        }
 }
 
 
 static void
 computer_probing_helper_done (HalDevice *d)
 {
+	/* check if this is may a laptop */
+	if (!hal_device_has_property (d, "system.formfactor") || 
+	    (strcmp (hal_device_property_get_string (d, "system.formfactor"), "laptop") != 0)) {
+		HAL_INFO (("Check if the machine is may a laptop ..."));
+		acpi_check_is_laptop("BATTERY");
+		acpi_check_is_laptop("LID");
+	}
 	/* if not set, set a default value */
 	if (!hal_device_has_property (d, "system.formfactor")) {
 		hal_device_property_set_string (d, "system.formfactor", "unknown");
 	}
+
+	/* will enqueue hotplug events for entire system */
+	HAL_INFO (("Synthesizing sysfs events..."));
+	coldplug_synthesize_events ();
+
+	HAL_INFO (("Synthesizing powermgmt events..."));
+	if (acpi_synthesize_hotplug_events ()) {
+		HAL_INFO (("ACPI capabilities found"));
+	} else if (pmu_synthesize_hotplug_events ()) {
+		HAL_INFO (("PMU capabilities found"));		
+	} else if (apm_synthesize_hotplug_events ()) {
+		HAL_INFO (("APM capabilities found"));		
+	} else {
+		HAL_INFO (("No powermgmt capabilities"));		
+	}
+	HAL_INFO (("Done synthesizing events"));
+
+        hald_done_synthesizing_coldplug = TRUE;
+
+	/* we try again to match again on computer, now we have done coldplug
+	 * and completed probing. In an ideal world, we would do this before
+	 * _and_ after the coldplug, but this seems to work well. */
 	di_search_and_merge (d, DEVICE_INFO_TYPE_INFORMATION);
 	di_search_and_merge (d, DEVICE_INFO_TYPE_POLICY);
 
@@ -642,7 +605,7 @@ get_openfirmware_entry(HalDevice *d, char *property, char *entry,
 	if (multivalue) {
 		gsize offset = 0;
 		while (offset < length) { 
-			hal_device_property_strlist_append(d, property, contents + offset);
+			hal_device_property_strlist_append(d, property, contents + offset, FALSE);
 			for (; offset < length - 1 && contents[offset] != '\0'; offset++)
 				;
 			offset++;
@@ -722,6 +685,38 @@ computer_probing_pm_is_supported_helper_done (HalDevice *d, guint32 exit_type,
         decode_dmi (d);
 }
 
+static void
+get_primary_videocard (HalDevice *d)
+{
+        GDir *dir;
+        const char *name;
+
+        dir = g_dir_open ("/sys/bus/pci/devices", 0, NULL);
+        if (dir == NULL)
+                goto out;
+        while ((name = g_dir_read_name (dir)) != NULL) {
+                int class;
+                char *path;
+                path = g_strdup_printf ("/sys/bus/pci/devices/%s", name);
+                if (hal_util_get_int_from_file (path, "class", &class, 0) && (class&0xffff00) == 0x030000 ) {
+                        int vendor, device;
+                        if (hal_util_get_int_from_file (path, "vendor", &vendor, 0) &&
+                            hal_util_get_int_from_file (path, "device", &device, 0)) {
+                                HAL_INFO (("got %x:%x as primary videocard", vendor, device));
+                                hal_device_property_set_int (d, "system.hardware.primary_video.vendor", vendor);
+                                hal_device_property_set_int (d, "system.hardware.primary_video.product", device);
+                                g_free (path);
+                                g_dir_close (dir);
+                                goto out;
+                        }
+                }
+                g_free (path);
+        }
+        g_dir_close (dir);
+out:
+        ;
+}
+
 void 
 osspec_probe (void)
 {
@@ -745,22 +740,6 @@ osspec_probe (void)
 	/* Let computer be in TDL while synthesizing all other events because some may write to the object */
 	hal_device_store_add (hald_get_tdl (), root);
 
-	/* will enqueue hotplug events for entire system */
-	HAL_INFO (("Synthesizing sysfs events..."));
-	coldplug_synthesize_events ();
-
-	HAL_INFO (("Synthesizing powermgmt events..."));
-	if (acpi_synthesize_hotplug_events ()) {
-		HAL_INFO (("ACPI capabilities found"));
-	} else if (pmu_synthesize_hotplug_events ()) {
-		HAL_INFO (("PMU capabilities found"));		
-	} else if (apm_synthesize_hotplug_events ()) {
-		HAL_INFO (("APM capabilities found"));		
-	} else {
-		HAL_INFO (("No powermgmt capabilities"));		
-	}
-	HAL_INFO (("Done synthesizing events"));
-
 	/*
 	 * Populate the powermgmt keys according to the kernel options.
 	 * NOTE: This may not mean the machine is able to suspend
@@ -769,6 +748,9 @@ osspec_probe (void)
 	 */
 	set_suspend_hibernate_keys (root);
 
+        /* set the vendor/product of primary video card */
+        get_primary_videocard (root);
+
         /* Try and set the suspend/hibernate keys using pm-is-supported
          */
         if (g_file_test ("/usr/bin/pm-is-supported", G_FILE_TEST_IS_EXECUTABLE)) {
