@@ -31,6 +31,7 @@
 #include <unistd.h>
 
 #include "../device_info.h"
+#include "../device_pm.h"
 #include "../hald_dbus.h"
 #include "../logger.h"
 #include "../util.h"
@@ -78,8 +79,6 @@ battery_refresh (HalDevice *d, PMUDevHandler *handler)
 	const char *path;
 	int flags;
 	int last_full;
-	int remaining_time;
-	int remaining_percentage;
 
 	path = hal_device_property_get_string (d, "linux.pmu_path");
 	if (path == NULL)
@@ -107,9 +106,7 @@ battery_refresh (HalDevice *d, PMUDevHandler *handler)
 		hal_device_property_set_bool (d, "battery.rechargeable.is_charging", flags & PMU_BATT_CHARGING);
 		/* we're discharging if, and only if, we are not plugged into the wall */
 		{
-			char buf[HAL_PATH_MAX];
-			snprintf (buf, sizeof (buf), "%s/pmu/info", get_hal_proc_path ());
-			hal_util_set_bool_elem_from_file (d, "battery.rechargeable.is_discharging", buf, "", 
+			hal_util_set_bool_elem_from_file (d, "battery.rechargeable.is_discharging", "/proc/pmu/info", "", 
 							  "AC Power", 0, "0", FALSE);
 		}
 
@@ -130,39 +127,13 @@ battery_refresh (HalDevice *d, PMUDevHandler *handler)
 		last_full = hal_device_property_get_int (d, "battery.charge_level.last_full");
 
 		/* TODO: could read some pmu file? */
-		remaining_time = util_compute_time_remaining (
-					hal_device_get_udi (d),
-					hal_device_property_get_int (d, "battery.charge_level.rate"),
-					current,
-					last_full,
-					hal_device_property_get_bool (d, "battery.rechargeable.is_discharging"),
-					hal_device_property_get_bool (d, "battery.rechargeable.is_charging"),
-					hal_device_property_get_bool (d, "battery.remaining_time.calculate_per_time"));
-		remaining_percentage = util_compute_percentage_charge (hal_device_get_udi (d), current, last_full);
-		/*
-		 * Only set keys if no error (signified with negative return value)
-		 * Scrict checking is needed to ensure that the values presented by HAL
-		 * are 100% acurate.
-		 */
-		if (remaining_time > 0)
-			hal_device_property_set_int (d, "battery.remaining_time", remaining_time);
-		else
-			hal_device_property_remove (d, "battery.remaining_time");
-		if (remaining_percentage > 0)
-			hal_device_property_set_int (d, "battery.charge_level.percentage", remaining_percentage);
-		else
-			hal_device_property_remove (d, "battery.charge_level.percentage");
+		device_pm_calculate_time (d);
+		device_pm_calculate_percentage (d);
 
 		device_property_atomic_update_end ();
 	} else {
 		device_property_atomic_update_begin ();
-		hal_device_property_remove (d, "battery.is_rechargeable");
-		hal_device_property_remove (d, "battery.rechargeable.is_charging");
-		hal_device_property_remove (d, "battery.rechargeable.is_discharging");
-		/*hal_device_property_remove (d, "battery.charge_level.unit");*/
-		hal_device_property_remove (d, "battery.charge_level.current");
-		hal_device_property_remove (d, "battery.charge_level.last_full");
-		hal_device_property_remove (d, "battery.charge_level.design");
+		device_pm_remove_optional_props (d);
 		device_property_atomic_update_end ();		
 	}
 
@@ -331,7 +302,6 @@ pmu_synthesize_hotplug_events (void)
 {
 	gboolean ret;
 	HalDevice *computer;
-	gchar path[HAL_PATH_MAX];
 	GError *error;
 	GDir *dir;
 	gboolean has_battery_bays;
@@ -355,19 +325,17 @@ pmu_synthesize_hotplug_events (void)
 	hal_device_property_set_string (computer, "power_management.type", "pmu");
 
 	/* AC Adapter */
-	snprintf (path, sizeof (path), "%s/pmu/info", get_hal_proc_path ());
-	pmu_synthesize_item (path, PMU_TYPE_AC_ADAPTER);
+	pmu_synthesize_item ("/proc/pmu/info", PMU_TYPE_AC_ADAPTER);
 
 	error = NULL;
-	snprintf (path, sizeof (path), "%s/pmu", get_hal_proc_path ());
-	dir = g_dir_open (path, 0, &error);
+	dir = g_dir_open ("/proc/pmu", 0, &error);
 	if (dir != NULL) {
 		const gchar *f;
 		while ((f = g_dir_read_name (dir)) != NULL) {
 			gchar buf[HAL_PATH_MAX];
 			int battery_num;
 
-			snprintf (buf, sizeof (buf), "%s/pmu/%s", get_hal_proc_path (), f);
+			snprintf (buf, sizeof (buf), "/proc/pmu/%s", f);
 			if (sscanf (f, "battery_%d", &battery_num) == 1) {
 				has_battery_bays = TRUE;
 				pmu_synthesize_item (buf, PMU_TYPE_BATTERY);
@@ -375,7 +343,7 @@ pmu_synthesize_hotplug_events (void)
 			
 		}
 	} else {
-		HAL_ERROR (("Couldn't open %s: %s", path, error->message));
+		HAL_ERROR (("Couldn't open /proc/pmu/info: %s", error->message));
 		g_error_free (error);
 	}
 
@@ -387,12 +355,10 @@ pmu_synthesize_hotplug_events (void)
 	 */
 	if (has_battery_bays) {
 		/* Add lid button */
-		snprintf (path, sizeof (path), "%s/pmu/info", get_hal_proc_path ());
-		pmu_synthesize_item (path, PMU_TYPE_LID_BUTTON);
+		pmu_synthesize_item ("/proc/pmu/info", PMU_TYPE_LID_BUTTON);
 
 		/* Add Laptop Panel */
-		snprintf (path, sizeof (path), "%s/pmu/info", get_hal_proc_path ());
-		pmu_synthesize_item (path, PMU_TYPE_LAPTOP_PANEL);
+		pmu_synthesize_item ("/proc/pmu/info", PMU_TYPE_LAPTOP_PANEL);
 	}
 
 	/* setup timer for things that we need to poll */
