@@ -58,6 +58,14 @@
 
 #include "linux_dvd_rw_utils.h"
 
+#if !defined(SG_FLAG_LUN_INHIBIT)
+# if defined(SG_FLAG_UNUSED_LUN_INHIBIT)
+#  define SG_FLAG_LUN_INHIBIT SG_FLAG_UNUSED_LUN_INHIBIT
+# else
+#  define SG_FLAG_LUN_INHIBIT 0
+# endif
+#endif
+
 typedef enum {
 	NONE = CGC_DATA_NONE,	// 3
 	READ = CGC_DATA_READ,	// 2
@@ -149,8 +157,8 @@ scsi_command_transport (ScsiCommand * cmd, Direction dir, void *buf,
 		errno = EIO;
 		ret = -1;
 		if (cmd->sg_io.masked_status & CHECK_CONDITION) {
-			CREAM_ON_ERRNO (cmd->sg_io.sbp);
-			ret = ERRCODE (cmd->sg_io.sbp);
+			CREAM_ON_ERRNO ((unsigned char*)cmd->sg_io.sbp);
+			ret = ERRCODE ((unsigned char*)cmd->sg_io.sbp);
 			if (ret == 0)
 				ret = -1;
 		}
@@ -229,6 +237,13 @@ get_dvd_r_rw_profile (int fd)
 		 * 0x1A: DVD+RW  
 		 * 0x2A: DVD+RW DL
 		 * 0x2B: DVD+R DL 
+		 * 0x40: BD-ROM
+		 * 0x41: BD-R SRM
+		 * 0x42: BR-R RRM
+		 * 0x43: BD-RE
+		 * 0x50: HD DVD-ROM
+		 * 0x51: HD DVD-R 
+		 * 0x52: HD DVD-Rewritable 
 		 */
 
 		switch (profile) {
@@ -248,6 +263,25 @@ get_dvd_r_rw_profile (int fd)
 			case 0x2B:
 				retval |= DRIVE_CDROM_CAPS_DVDPLUSRDL;
 				break;
+			case 0x40:
+				retval |= DRIVE_CDROM_CAPS_BDROM;
+				break;
+			case 0x41:
+			case 0x42:
+				retval |= DRIVE_CDROM_CAPS_BDR;
+				break;
+			case 0x43:
+				retval |= DRIVE_CDROM_CAPS_BDRE;
+				break;
+			case 0x50:
+				retval |= DRIVE_CDROM_CAPS_HDDVDROM;
+				break;
+			case 0x51:
+				retval |= DRIVE_CDROM_CAPS_HDDVDR;
+				break;
+			case 0x52:
+				retval |= DRIVE_CDROM_CAPS_HDDVDRW;
+				break;
 			default:
 				break;
 		}
@@ -664,6 +698,7 @@ get_disc_capacity_dvdr_from_type (int fd,
 	case 0x11:		/* DVD-R */
 	case 0x1B:		/* DVD+R */
 	case 0x2B:		/* DVD+R Double Layer */
+	case 0x41:		/* BD-R SRM */
 
 		/* READ TRACK INFORMATION */
 		scsi_command_init (cmd, 0, 0x52);
@@ -689,6 +724,13 @@ get_disc_capacity_dvdr_from_type (int fd,
 
 		retval = 0;
 		break;
+	case 0x43:		/* BD-RE */
+		/* Pull the formatted capacity */
+		blocks  = formats [4 + 0] << 24;
+		blocks |= formats [4 + 1] << 16;
+		blocks |= formats [4 + 2] << 8;
+		blocks |= formats [4 + 3];
+		break;
 	default:
 		blocks = 0;
 		break;
@@ -731,6 +773,8 @@ get_disc_capacity_for_type (int fd,
 	case 0x2B:
 	case 0x1A:
 	case 0x12:
+	case 0x41:
+	case 0x43:
 		retval = get_disc_capacity_dvdr_from_type (fd, type, size);
 		break;
 	default:
