@@ -16,7 +16,14 @@
 #define DRIVE_CDROM_CAPS_DVDPLUSRW    4
 #define DRIVE_CDROM_CAPS_DVDPLUSRWDL  8
 #define DRIVE_CDROM_CAPS_DVDPLUSRDL  16
+#define DRIVE_CDROM_CAPS_BDROM       32
+#define DRIVE_CDROM_CAPS_BDR         64
+#define DRIVE_CDROM_CAPS_BDRE       128
+#define DRIVE_CDROM_CAPS_HDDVDROM   256
+#define DRIVE_CDROM_CAPS_HDDVDR     512
+#define DRIVE_CDROM_CAPS_HDDVDRW   1024
 
+ 
 int get_dvd_r_rw_profile (int fd);
 int get_read_write_speed (int fd, int *read_speed, int *write_speed, char **write_speeds);
 int get_disc_capacity_for_type (int fd, int type, guint64 *capacity);
