@@ -33,10 +33,12 @@
 
 #include <glib.h>
 #include <libhal.h>
+#include <polkit/polkit.h>
 
 /* How this works (or "An introduction to this code")
  *
  * - all ACL's granted by this tool is kept in /var/lib/hal/acl-list
+ *   See below for the format.
  *
  * - every time tool is launched we read this file and keep each line
  *   as an ACLCurrent instance. These are kept in a list.
@@ -52,6 +54,7 @@
  *     - ACL's to be added are appended to the list and add -> TRUE
  *   - we then compute the argument vector to setfacl(1) for adding /
  *     removing ACL's
+ *   - we emit signals ACLAdded and ACLRemoved
  *   - if setfacl(1) succeeds (rc == 0) then we write the new acl-current-list
  *
  * Notably, the HAL daemon will invoke us with --reconfigure on every
@@ -83,24 +86,24 @@
  * of ACL's that have been set by HAL and as such are currently
  * applied
  *
- *   <device-file>    <type>    <uid-or-gid>
+ *   <device-file>    <udi>    <type>    <uid-or-gid>
  *
  * where <type>='u'|'g' for respectively uid and gid (the spacing represent tabs).
  *
  * Example:
  *
- *   /dev/snd/controlC0    u    500
- *   /dev/snd/controlC0    u    501
- *   /dev/snd/controlC0    g    1001
+ *   /dev/snd/controlC0    /org/freedesktop/Hal/devices/pci_8086_27d8_alsa_control__1    u    500
+ *   /dev/snd/controlC0    /org/freedesktop/Hal/devices/pci_8086_27d8_alsa_control__1    u    501
+ *   /dev/snd/controlC0    /org/freedesktop/Hal/devices/pci_8086_27d8_alsa_control__1    g    1001
  */
 typedef struct ACLCurrent_s {
+        char *udi;
 	char *device;
 	int type;
 	union {
 		uid_t uid;
 		gid_t gid;
 	} v;
-
 	gboolean remove;
 	gboolean add;
 } ACLCurrent;
@@ -111,9 +114,41 @@ enum {
 	HAL_ACL_GID
 };
 
+static PolKitContext *pk_context = NULL;
+
+static LibHalContext *hal_ctx = NULL;
+
+static gboolean in_device_added = FALSE;
+
+static gboolean
+ensure_hal_ctx (void)
+{
+        gboolean ret;
+	DBusError error;
+
+        ret = FALSE;
+        if (hal_ctx != NULL) {
+                ret = TRUE;
+                goto out;
+        }
+
+	dbus_error_init (&error);
+	if ((hal_ctx = libhal_ctx_init_direct (&error)) == NULL) {
+		printf ("%d: Cannot connect to hald: %s: %s\n", getpid (), error.name, error.message);
+		LIBHAL_FREE_DBUS_ERROR (&error);
+                goto out;
+	}
+
+        ret = TRUE;
+
+out:
+        return ret;
+}
+
 static void
 hal_acl_free (ACLCurrent *ha)
 {
+        g_free (ha->udi);
 	g_free (ha->device);
 	g_free (ha);
 }
@@ -135,6 +170,7 @@ acl_apply_changes (GSList *new_acl_list, gboolean only_update_acllist, gboolean
 	gboolean ret;
 	GError *error = NULL;
 	int exit_status;
+        gboolean messages_were_sent = FALSE;
 
 	ret = FALSE;
 
@@ -156,7 +192,6 @@ acl_apply_changes (GSList *new_acl_list, gboolean only_update_acllist, gboolean
 						(ha->type == HAL_ACL_UID) ? 'u' : 'g',
 						(ha->type == HAL_ACL_UID) ? ha->v.uid : ha->v.gid,
 						ha->device);
-			continue;
 		}
 
 		if (ha->add) {
@@ -167,13 +202,17 @@ acl_apply_changes (GSList *new_acl_list, gboolean only_update_acllist, gboolean
 						(ha->type == HAL_ACL_UID) ? 'u' : 'g',
 						(ha->type == HAL_ACL_UID) ? ha->v.uid : ha->v.gid,
 						ha->device);
+
 		}
 
-		g_string_append_printf (str, 
-					"%s\t%c\t%d\n",
-					ha->device,
-					(ha->type == HAL_ACL_UID) ? 'u' : 'g',
-					(ha->type == HAL_ACL_UID) ? ha->v.uid : ha->v.gid);
+                if (!ha->remove) {
+                        g_string_append_printf (str, 
+                                                "%s\t%s\t%c\t%d\n",
+                                                ha->device,
+                                                ha->udi,
+                                                (ha->type == HAL_ACL_UID) ? 'u' : 'g',
+                                                (ha->type == HAL_ACL_UID) ? ha->v.uid : ha->v.gid);
+                }
 	}
 	new_acl_file_contents = g_string_free (str, FALSE);
 
@@ -224,6 +263,79 @@ acl_apply_changes (GSList *new_acl_list, gboolean only_update_acllist, gboolean
 			     strlen (new_acl_file_contents),
 			     NULL);
 
+        /* now that we've added/removed ACL's: emit D-Bus signals.. we need to do this _after_ 
+         * having updated the ACL - to avoid possible race conditions 
+         */
+        messages_were_sent = FALSE;
+	for (i = new_acl_list; i != NULL; i = g_slist_next (i)) {
+		ACLCurrent *ha = (ACLCurrent *) i->data;
+
+                /* emit signal ACLGranted or ACLRemoved - but avoid doing it on device add because the device
+                 * object doesn't exist just yet
+                 */
+                if (ha->type == HAL_ACL_UID && ha->udi != NULL && (ha->add || ha->remove) && !in_device_added) {
+                        DBusMessage *message;
+                        DBusConnection *connection;
+
+                        printf ("Emmitting %s for udi %s, uid %d\n", 
+                                ha->add ? "ACLAdded" : "ACLRemoved",
+                                ha->udi,
+                                ha->v.uid);
+
+                        if (!ensure_hal_ctx ()) {
+				printf ("%d: cannot get libhal context\n", getpid ());
+                                goto no_dbus_signal;
+                        }
+
+                        connection = libhal_ctx_get_dbus_connection (hal_ctx);
+                        if (connection == NULL) {
+				printf ("%d: cannot get D-Bus connection\n", getpid());
+                                goto no_dbus_signal;
+                        }
+
+                        message = dbus_message_new_signal (ha->udi,
+                                                           "org.freedesktop.Hal.Device.AccessControl",
+                                                           ha->add ? "ACLAdded" : "ACLRemoved");
+                        if (message == NULL) {
+				printf ("%d: cannot create message\n", getpid());
+                                goto no_dbus_signal;
+                        }
+
+                        if (!dbus_message_append_args (message, 
+                                                       DBUS_TYPE_UINT32, &(ha->v.uid),
+                                                       DBUS_TYPE_INVALID)) {
+				printf ("%d: cannot append to message\n", getpid());
+                                dbus_message_unref (message);
+                                goto no_dbus_signal;
+                        }
+
+                        if (!dbus_connection_send (connection, message, NULL)) {
+				printf ("%d: cannot send message\n", getpid());
+                                dbus_message_unref (message);
+                                goto no_dbus_signal;
+                        }
+                        dbus_message_unref (message);
+
+                        messages_were_sent = TRUE;
+                }
+
+        no_dbus_signal:
+                ;
+        }
+
+        /* apparently we need to flush the signals - otherwise they
+         * may be lost because we're exiting right after this
+         */
+        if (messages_were_sent) {
+                if (ensure_hal_ctx ()) {
+                        DBusConnection *connection;
+                        connection = libhal_ctx_get_dbus_connection (hal_ctx);
+                        if (connection != NULL) {
+                                dbus_connection_flush (connection);
+                        }
+                }
+        }
+
 	ret = TRUE;
 
 out:
@@ -258,27 +370,45 @@ get_current_acl_list (GSList **l)
 		ACLCurrent *ha;
 		char **val;
 		char *endptr;
+                gboolean old_format = FALSE;
+                int n;
+
+                /* supporting the old format is important; because at hald startup we call hal-acl-tool --remove-all */
 
 		ha = g_new0 (ACLCurrent, 1);
 		val = g_strsplit(buf, "\t", 0);
-		if (g_strv_length (val) != 3) {
-			printf ("Line is malformed: '%s'\n", buf);
-			g_strfreev (val);
-			goto out;
-		}
+		if (g_strv_length (val) == 3) {
+			printf ("Line is from old format: '%s'\n", buf);
+                        old_format = TRUE;
+		} else {
+                        if (g_strv_length (val) != 4) {
+                                printf ("Line is malformed: '%s'\n", buf);
+                                g_strfreev (val);
+                                goto out;
+                        }
+                }
 
-		ha->device = g_strdup (val[0]);
-		if (strcmp (val[1], "u") == 0) {
+                n = 0;
+
+		ha->device = g_strdup (val[n++]);
+
+                if (old_format) {
+                        ha->udi = NULL;
+                } else {
+                        ha->udi = g_strdup (val[n++]);
+                }
+
+		if (strcmp (val[n], "u") == 0) {
 			ha->type = HAL_ACL_UID;
-			ha->v.uid = strtol (val[2], &endptr, 10);
+			ha->v.uid = strtol (val[n + 1], &endptr, 10);
 			if (*endptr != '\0' && *endptr != '\n') {
 				printf ("Line is malformed: '%s'\n", buf);
 				g_strfreev (val);
 				goto out;
 			}
-		} else if (strcmp (val[1], "g") == 0) {
+		} else if (strcmp (val[n], "g") == 0) {
 			ha->type = HAL_ACL_GID;
-			ha->v.gid = strtol (val[2], &endptr, 10);
+			ha->v.gid = strtol (val[n + 1], &endptr, 10);
 			if (*endptr != '\0' && *endptr != '\n') {
 				printf ("Line is malformed: '%s'\n", buf);
 				g_strfreev (val);
@@ -336,6 +466,7 @@ visit_seats_and_sessions (SeatSessionVisitor visitor_cb, gpointer user_data)
 	/* for all seats */
 	for (i = 0; seats[i] != NULL; i++) {
 		char *seat = seats[i];
+                char *seat_id;
 		char **sessions;
 		int num_sessions_on_seat;
 
@@ -349,11 +480,13 @@ visit_seats_and_sessions (SeatSessionVisitor visitor_cb, gpointer user_data)
 		sessions = g_strsplit (s, "\t", 0);
 		num_sessions_on_seat = g_strv_length (sessions);
 
-		visitor_cb (seat, num_sessions_on_seat, NULL, 0, FALSE, FALSE, user_data);
+                seat_id = g_strdup_printf ("/org/freedesktop/ConsoleKit/%s", seat);
+		visitor_cb (seat_id, num_sessions_on_seat, NULL, 0, FALSE, FALSE, user_data);
 
 		/* for all sessions on seat */
 		for (j = 0; sessions[j] != NULL; j++) {
 			char *session = sessions[j];
+                        char *session_id;
 			gboolean session_is_local;
 			gboolean session_is_active;
 			uid_t session_uid;
@@ -390,11 +523,15 @@ visit_seats_and_sessions (SeatSessionVisitor visitor_cb, gpointer user_data)
 				goto out;
 			}
 
-			visitor_cb (seat, num_sessions_on_seat, 
-				    session, session_uid, session_is_local, session_is_active, user_data);
+                        session_id = g_strdup_printf ("/org/freedesktop/ConsoleKit/%s", session);
 
+			visitor_cb (seat_id, num_sessions_on_seat, 
+				    session_id, session_uid, session_is_local, session_is_active, user_data);
+
+                        g_free (session_id);
 		}
 		g_strfreev (sessions);
+                g_free (seat_id);
 	}
 	g_strfreev (seats);
 
@@ -409,14 +546,7 @@ typedef struct {
 	/* identifying the device */
 	char *udi;      /* HAL UDI of device */
 	char *device;   /* device file */
-
-	/* policy for how to apply ACL's (must be set by the caller prior to visiting the device) */
-
-	/* access is granted to any session on a local seat */
-	gboolean grant_to_local_seat;
-
-	/* access is granted only to active sessions on local seats */
-	gboolean grant_to_local_seat_active_only;
+        char *type;     /* type of device, access_control.type - used by PolicyKit */
 
 	/* will be set by the visitor */
 	GSList *uid;    /* list of uid's (int) that should have access to this device */
@@ -568,11 +698,18 @@ acl_for_device_set_device (ACLForDevice *afd, const char *device)
 	afd->device = g_strdup (device);
 }
 
+static void
+acl_for_device_set_type (ACLForDevice *afd, const char *type)
+{
+	afd->type = g_strdup (type);
+}
+
 static void
 acl_for_device_free (ACLForDevice* afd)
 {
 	g_free (afd->udi);
 	g_free (afd->device);
+	g_free (afd->type);
 	g_slist_free (afd->uid);
 	g_slist_free (afd->gid);
 	g_free (afd);
@@ -607,6 +744,11 @@ acl_device_added_visitor (const char *seat_id,
 	 */
 	for (i = afd_list; i != NULL; i = g_slist_next (i)) {
 		ACLForDevice *afd = (ACLForDevice *) i->data;
+                PolKitResult pk_result;
+                PolKitSeat *pk_seat;
+                PolKitSession *pk_session;
+                PolKitAction *pk_action;
+                char *priv_name;
 
 		if (session_id == NULL) {
 			/* we only grant access to sessions - someone suggested that if a device is tied
@@ -619,26 +761,33 @@ acl_device_added_visitor (const char *seat_id,
 			continue;
 		}
 
+                pk_seat = polkit_seat_new ();
+                polkit_seat_set_ck_objref (pk_seat, seat_id);
 
-		/* apply the policy defined by grant_to_local_seat and grant_to_local_seat_active_only */
+                pk_session = polkit_session_new ();
+                polkit_session_set_seat (pk_session, pk_seat);
+                polkit_seat_unref (pk_seat);
+                polkit_session_set_ck_objref (pk_session, session_id);
+                polkit_session_set_uid (pk_session, session_uid);
+                polkit_session_set_ck_is_active (pk_session, session_is_active);
+                polkit_session_set_ck_is_local (pk_session, session_is_local);
+                /* TODO: FIXME: polkit_session_set_ck_remote_host (pk_session, );*/ 
 
-		/* we only grant access to local seats... */
-		if (!session_is_local)
-			continue;
+                pk_action = polkit_action_new();
+                priv_name = g_strdup_printf ("org.freedesktop.hal.device-access.%s", afd->type);
+                polkit_action_set_action_id (pk_action, priv_name);
+                g_free (priv_name);
 
-		/* don't bother giving ACL's to root - he's almighty anyway */
-		if (session_uid == 0)
-			continue;
-
-		if (afd->grant_to_local_seat)
+                /* Now ask PolicyKit if the given session should have access */
+                pk_result = polkit_context_can_session_do_action (pk_context, 
+                                                                  pk_action,
+                                                                  pk_session);
+                if (pk_result == POLKIT_RESULT_YES) {
 			afd_grant_to_uid (afd, session_uid);
-		else {
-			if (afd->grant_to_local_seat_active_only) {
-				if (session_is_active) {
-					afd_grant_to_uid (afd, session_uid);
-				}
-			}
-		}
+                }
+
+                polkit_action_unref (pk_action);
+                polkit_session_unref (pk_session);
 	}
 
 }
@@ -733,11 +882,19 @@ acl_compute_changes (GSList *afd_list, gboolean only_update_acllist)
 		 */
 		for (j = afd->uid; j != NULL; j = g_slist_next (j)) {
 			ACLCurrent *ha;
+                        uid_t uid;
+
+                        uid = GPOINTER_TO_INT (j->data);
+                        /* never grant ACL's to the super user */
+                        if (uid == 0)
+                                continue;
+
 			ha = g_new0 (ACLCurrent, 1);
 			ha->add = TRUE;
 			ha->device = g_strdup (afd->device);
+                        ha->udi = g_strdup (afd->udi);
 			ha->type = HAL_ACL_UID;
-			ha->v.uid = GPOINTER_TO_INT (j->data);
+			ha->v.uid = uid;
 			current_acl_list = g_slist_prepend (current_acl_list, ha);
 		}
 		for (j = afd->gid; j != NULL; j = g_slist_next (j)) {
@@ -745,6 +902,7 @@ acl_compute_changes (GSList *afd_list, gboolean only_update_acllist)
 			ha = g_new0 (ACLCurrent, 1);
 			ha->add = TRUE;
 			ha->device = g_strdup (afd->device);
+                        ha->udi = g_strdup (afd->udi);
 			ha->type = HAL_ACL_GID;
 			ha->v.gid = GPOINTER_TO_INT (j->data);
 			current_acl_list = g_slist_prepend (current_acl_list, ha);
@@ -778,9 +936,12 @@ acl_device_added (void)
 	char *s;
 	char *udi;
 	char *device;
+	char *type;
 	GSList *afd_list = NULL;
 	ACLForDevice *afd = NULL;
 
+        in_device_added = TRUE;
+
 	/* we can avoid round-trips to the HAL daemon by using what's in the environment */
 
 	if ((udi = getenv ("UDI")) == NULL)
@@ -789,17 +950,15 @@ acl_device_added (void)
 	if ((device = getenv ("HAL_PROP_ACCESS_CONTROL_FILE")) == NULL)
 		goto out;
 
+	if ((type = getenv ("HAL_PROP_ACCESS_CONTROL_TYPE")) == NULL)
+		goto out;
+
 	afd = acl_for_device_new (udi);
 	acl_for_device_set_device (afd, device);
+	acl_for_device_set_type (afd, type);
 	afd_list = g_slist_prepend (NULL, afd);
 
 	/* get ACL granting policy from HAL properties */
-	if ((s = getenv ("HAL_PROP_ACCESS_CONTROL_GRANT_LOCAL_SESSION")) != NULL) {
-		afd->grant_to_local_seat = (strcmp (s, "true") == 0);
-	}
-	if ((s = getenv ("HAL_PROP_ACCESS_CONTROL_GRANT_LOCAL_ACTIVE_SESSION")) != NULL) {
-		afd->grant_to_local_seat_active_only = (strcmp (s, "true") == 0);
-	}
 	if ((s = getenv ("HAL_PROP_ACCESS_CONTROL_GRANT_USER")) != NULL) {
 		char **sv;
 		sv = g_strsplit (s, "\t", 0);
@@ -879,18 +1038,14 @@ acl_reconfigure_all (void)
 	int num_devices;
 	char **udis;
 	DBusError error;
-	LibHalContext *hal_ctx;
 	GSList *afd_list = NULL;
 
 	printf ("%d: reconfiguring all ACL's\n", getpid ());
 
-	dbus_error_init (&error);
-	if ((hal_ctx = libhal_ctx_init_direct (&error)) == NULL) {
-		printf ("%d: Cannot connect to hald: %s: %s\n", getpid (), error.name, error.message);
-		LIBHAL_FREE_DBUS_ERROR (&error);
-		goto out;
-	}
+        if (!ensure_hal_ctx ())
+                goto out;
 
+	dbus_error_init (&error);
 	if ((udis = libhal_find_device_by_capability (hal_ctx, "access_control", &num_devices, &error)) == NULL) {
 		printf ("%d: Cannot get list of devices of capability 'acl'\n", getpid ());
 		goto out;
@@ -900,6 +1055,7 @@ acl_reconfigure_all (void)
 		LibHalPropertySet *props;
 		LibHalPropertySetIterator psi;
 		char *device = NULL;
+                char *type = NULL;
 		ACLForDevice *afd;
 		char **sv;
 
@@ -916,10 +1072,8 @@ acl_reconfigure_all (void)
 			key = libhal_psi_get_key (&psi);
 			if (strcmp (key, "access_control.file") == 0) {
 				device = libhal_psi_get_string (&psi);
-			} else if (strcmp (key, "access_control.grant_local_session") == 0) {
-				afd->grant_to_local_seat = libhal_psi_get_bool (&psi);
-			} else if (strcmp (key, "access_control.grant_local_active_session") == 0) {
-				afd->grant_to_local_seat_active_only = libhal_psi_get_bool (&psi);
+			} else if (strcmp (key, "access_control.type") == 0) {
+				type = libhal_psi_get_string (&psi);
 			} else if (strcmp (key, "access_control.grant_user") == 0) {
 				sv = libhal_psi_get_strlist (&psi);
 				afd_grant_to_uid_from_userlist (afd, sv);
@@ -932,12 +1086,26 @@ acl_reconfigure_all (void)
 
 		if (device == NULL) {
 			printf ("%d: access_control.file not set for '%s'\n", getpid (), udis[i]);
-			goto out;
-		} else {
-			acl_for_device_set_device (afd, device);
-			afd_list = g_slist_prepend (afd_list, afd);
+                        if (type != NULL)
+                                libhal_free_string (type);
+                        acl_for_device_free (afd);
+                        goto skip;
 		}
 
+		if (type == NULL) {
+			printf ("%d: access_control.type not set for '%s'\n", getpid (), udis[i]);
+                        if (device != NULL)
+                                libhal_free_string (device);
+                        if (type != NULL)
+                                libhal_free_string (type);
+                        acl_for_device_free (afd);
+                        goto skip;
+		}
+
+                acl_for_device_set_device (afd, device);
+                acl_for_device_set_type (afd, type);
+                afd_list = g_slist_prepend (afd_list, afd);
+        skip:
 		libhal_free_property_set (props);
 	}
 	libhal_free_string_array (udis);
@@ -989,7 +1157,7 @@ acl_lock (void)
 
 	printf ("%d: attempting to get lock on " PACKAGE_LOCALSTATEDIR "/lib/hal/acl-list\n", getpid ());
 
-	lock_acl_fd = open (PACKAGE_LOCALSTATEDIR "/lib/hal/acl-list", O_CREAT | O_RDWR);
+	lock_acl_fd = open (PACKAGE_LOCALSTATEDIR "/lib/hal/acl-list", O_CREAT | O_RDWR, 0644);
 	if (lock_acl_fd < 0) {
 		printf ("%d: error opening/creating " PACKAGE_LOCALSTATEDIR "/lib/hal/acl-list\n", getpid ());
 		return FALSE;
@@ -1025,10 +1193,11 @@ acl_unlock (void)
 	printf ("%d: released lock on " PACKAGE_LOCALSTATEDIR "/lib/hal/acl-list\n", getpid ());
 }
 
-
 int
 main (int argc, char *argv[])
 {
+        PolKitError *p_error;
+
 	if (argc != 2) {
 		printf ("hal-acl-tool should only be invoked by hald\n");
 		goto out;
@@ -1038,6 +1207,13 @@ main (int argc, char *argv[])
 		goto out;
 	}
 
+        p_error = NULL;
+        pk_context = polkit_context_new ();
+        if (!polkit_context_init (pk_context, &p_error)) {
+                printf ("Could not init PolicyKit context: %s\n", polkit_error_get_error_message (p_error));
+                goto out;
+        }
+
 	if (strcmp (argv[1], "--add-device") == 0) {
 		acl_device_added ();
 	} else if (strcmp (argv[1], "--remove-device") == 0) {
