@@ -36,9 +36,6 @@
 
 #include <libhal.h>
 #include <libhal-storage.h>
-#ifdef HAVE_POLKIT
-#include <libpolkit.h>
-#endif
 
 #include "hal-storage-shared.h"
 
@@ -77,9 +74,6 @@ main (int argc, char *argv[])
 	DBusError error;
 	LibHalContext *hal_ctx = NULL;
 	DBusConnection *system_bus = NULL;
-#ifdef HAVE_POLKIT
-	LibPolKitContext *pol_ctx = NULL;
-#endif
 	char *invoked_by_uid;
 	char *invoked_by_syscon_name;
 	int i;
@@ -117,13 +111,6 @@ main (int argc, char *argv[])
 		}
 		usage ();
 	}
-#ifdef HAVE_POLKIT
-	pol_ctx = libpolkit_new_context (system_bus);
-	if (pol_ctx == NULL) {
-		printf ("Cannot get libpolkit context\n");
-		unknown_closetray_error ("Cannot get libpolkit context");
-	}
-#endif
 
 	/* read from stdin */
 	if (strlen (fgets (closetray_options, sizeof (closetray_options), stdin)) > 0)
@@ -160,9 +147,6 @@ main (int argc, char *argv[])
 
 	/* use handle_eject() with the closetray option */
 	handle_eject (hal_ctx, 
-#ifdef HAVE_POLKIT
-		      pol_ctx, 
-#endif
 		      libhal_drive_get_udi (drive),
 		      drive,
 		      libhal_drive_get_device_file (drive),
