@@ -51,9 +51,6 @@
 
 #include <libhal.h>
 #include <libhal-storage.h>
-#ifdef HAVE_POLKIT
-#include <libpolkit.h>
-#endif
 
 #include "hal-storage-shared.h"
 
@@ -145,10 +142,10 @@ cannot_remount (const char *device)
 
 #ifdef HAVE_POLKIT
 static void
-permission_denied_privilege (const char *privilege, const char *uid)
+permission_denied_action (const char *action, const char *result)
 {
 	fprintf (stderr, "org.freedesktop.Hal.Device.PermissionDeniedByPolicy\n");
-	fprintf (stderr, "%s refused uid %s\n", privilege, uid);
+	fprintf (stderr, "%s %s <-- (action, result)\n", action, result);
 	exit (1);
 }
 #endif
@@ -443,9 +440,6 @@ map_fstype (const char *fstype)
 
 static void
 handle_mount (LibHalContext *hal_ctx, 
-#ifdef HAVE_POLKIT
-	      LibPolKitContext *pol_ctx, 
-#endif
 	      const char *udi,
 	      LibHalVolume *volume, LibHalDrive *drive, const char *device, 
 	      const char *invoked_by_uid, const char *invoked_by_syscon_name)
@@ -468,12 +462,8 @@ handle_mount (LibHalContext *hal_ctx,
 	GString *mount_option_str;
 	gboolean pol_is_fixed;
 	gboolean pol_change_uid;
-	char *privilege;
+	char *action;
 	gboolean is_remount;
-#ifdef HAVE_POLKIT
-	gboolean allowed_by_privilege;
-	gboolean is_temporary_privilege;
-#endif
 	gboolean explicit_mount_point_given;
 	const char *end;
 #ifdef __FreeBSD__
@@ -728,83 +718,17 @@ handle_mount (LibHalContext *hal_ctx,
 	 * (since these drives normally use vfat)
 	 */
 	if (volume != NULL) {
-		/* don't consider uid= on vfat, iso9660, udf change-uid for the purpose of policy
+		/* don't consider uid= on vfat, iso9660, hfs and udf change-uid for the purpose of policy
 		 * (since these doesn't contain uid/gid bits) 
 		 */
 		if (strcmp (libhal_volume_get_fstype (volume), "vfat") != 0 &&
 		    strcmp (libhal_volume_get_fstype (volume), "iso9660") != 0 &&
+		    strcmp (libhal_volume_get_fstype (volume), "hfs") != 0 &&
 		    strcmp (libhal_volume_get_fstype (volume), "udf") != 0) {
 			pol_change_uid = wants_to_change_uid;
 		}
 	}
 
-	if (pol_is_fixed) {
-		if (pol_change_uid) {
-			privilege = "hal-storage-fixed-mount-all-options";
-		} else {
-			privilege = "hal-storage-fixed-mount";
-		}
-	} else {
-		if (pol_change_uid) {
-			privilege = "hal-storage-removable-mount-all-options";
-		} else {
-			privilege = "hal-storage-removable-mount";
-		}
-	}
-
-#ifdef DEBUG
-	printf ("using privilege %s for uid %s, system_bus_connection %s\n", privilege, invoked_by_uid, 
-		invoked_by_syscon_name);
-#endif
-
-#ifdef HAVE_POLKIT
-        if (invoked_by_syscon_name != NULL) {
-                if (libpolkit_is_uid_allowed_for_privilege (pol_ctx, 
-                                                            invoked_by_syscon_name,
-                                                            invoked_by_uid,
-                                                            privilege,
-                                                            udi,
-                                                            &allowed_by_privilege,
-                                                            &is_temporary_privilege,
-                                                            NULL) != LIBPOLKIT_RESULT_OK) {
-                        printf ("cannot lookup privilege\n");
-                        unknown_error ("Cannot lookup privilege from PolicyKit");
-                }
-
-                if (!allowed_by_privilege) {
-                        printf ("caller don't possess privilege\n");
-                        permission_denied_privilege (privilege, invoked_by_uid);
-                }
-        }
-#endif
-
-#ifdef DEBUG
-	printf ("passed privilege\n");
-#endif
-
-	if (!is_remount) {
-		/* create directory */
-		if (g_mkdir (mount_dir, 0700) != 0) {
-			printf ("Cannot create '%s'\n", mount_dir);
-			unknown_error ("Cannot create mount directory");
-		}
-		
-#ifdef __FreeBSD__
-		calling_uid = (uid_t) strtol (invoked_by_uid, (char **) NULL, 10);
-		pw = getpwuid (calling_uid);
-		if (pw != NULL) {
-			calling_gid = pw->pw_gid;
-		} else {
-			calling_gid = 0;
-		}
-		if (chown (mount_dir, calling_uid, calling_gid) != 0) {
-			printf ("Cannot chown '%s' to uid: %d, gid: %d\n", mount_dir,
-				calling_uid, calling_gid);
-			g_rmdir (mount_dir);
-			unknown_error ("Failed to chown mount directory");
-		}
-#endif
-	}
 
 	char *mount_option_commasep = NULL;
 	char *mount_do_fstype = "auto";
@@ -839,6 +763,86 @@ handle_mount (LibHalContext *hal_ctx,
 	args[na++] = mount_dir;
 	args[na++] = NULL;
 
+	if (pol_is_fixed) {
+		if (pol_change_uid) {
+			action = NULL; /* "hal-storage-mount-fixed-extra-options"; TODO: rethink */
+		} else {
+			action = "org.freedesktop.hal.storage.mount-fixed";
+		}
+	} else {
+		if (pol_change_uid) {
+			action = NULL; /* "hal-storage-mount-removable-extra-options"; TODO: rethink "extra-options" */
+		} else {
+			action = "org.freedesktop.hal.storage.mount-removable";
+		}
+	}
+        if (action == NULL) {
+                unknown_error ("TODO: have to rethink extra options");
+        }
+
+#ifdef DEBUG
+	printf ("using action %s for uid %s, system_bus_connection %s\n", action, invoked_by_uid, 
+		invoked_by_syscon_name);
+#endif
+
+#ifdef HAVE_POLKIT
+        if (invoked_by_syscon_name != NULL) {
+                char *polkit_result;
+                char *action_params[] = {
+                        "fstype", "",
+                        "mount-point", "",
+                        "mount-options", "",
+                        NULL};
+
+                action_params[1] = mount_do_fstype;
+                action_params[3] = mount_dir;
+                action_params[5] = mount_option_commasep;
+
+                dbus_error_init (&error);
+                polkit_result = libhal_device_is_caller_privileged (hal_ctx,
+                                                                    udi,
+                                                                    action,
+                                                                    invoked_by_syscon_name,
+                                                                    &error);
+                if (polkit_result == NULL){
+                        unknown_error ("IsCallerPrivileged() failed");
+                }
+                if (strcmp (polkit_result, "yes") != 0) {
+                        permission_denied_action (action, polkit_result);
+                }
+                libhal_free_string (polkit_result);
+        }
+#endif
+
+#ifdef DEBUG
+	printf ("passed privilege\n");
+#endif
+
+	if (!is_remount) {
+		/* create directory */
+		if (g_mkdir (mount_dir, 0700) != 0) {
+			printf ("Cannot create '%s'\n", mount_dir);
+			unknown_error ("Cannot create mount directory");
+		}
+		
+#ifdef __FreeBSD__
+		calling_uid = (uid_t) strtol (invoked_by_uid, (char **) NULL, 10);
+		pw = getpwuid (calling_uid);
+		if (pw != NULL) {
+			calling_gid = pw->pw_gid;
+		} else {
+			calling_gid = 0;
+		}
+		if (chown (mount_dir, calling_uid, calling_gid) != 0) {
+			printf ("Cannot chown '%s' to uid: %d, gid: %d\n", mount_dir,
+				calling_uid, calling_gid);
+			g_rmdir (mount_dir);
+			unknown_error ("Failed to chown mount directory");
+		}
+#endif
+	}
+
+
 	/* TODO FIXME XXX HACK: OK, so we should rewrite the options in /media/.hal-mtab .. 
 	 *                      but it doesn't really matter much at this point */
 	if (!is_remount) {
@@ -994,9 +998,6 @@ main (int argc, char *argv[])
 	DBusError error;
 	LibHalContext *hal_ctx = NULL;
 	DBusConnection *system_bus = NULL;
-#ifdef HAVE_POLKIT
-	LibPolKitContext *pol_ctx = NULL;
-#endif
 	char *invoked_by_uid;
 	char *invoked_by_syscon_name;
 
@@ -1034,13 +1035,6 @@ main (int argc, char *argv[])
 		}
 		usage ();
 	}
-#ifdef HAVE_POLKIT
-	pol_ctx = libpolkit_new_context (system_bus);
-	if (pol_ctx == NULL) {
-		printf ("Cannot get libpolkit context\n");
-		unknown_error ("Cannot get libpolkit context");
-	}
-#endif
 
 	volume = libhal_volume_from_udi (hal_ctx, udi);
 	if (volume == NULL) {
@@ -1051,9 +1045,6 @@ main (int argc, char *argv[])
 			usage ();
 		} else {
 			handle_mount (hal_ctx, 
-#ifdef HAVE_POLKIT
-				      pol_ctx, 
-#endif
 				      udi, NULL, drive, device, invoked_by_uid, 
 				      invoked_by_syscon_name);
 		}
@@ -1071,9 +1062,6 @@ main (int argc, char *argv[])
 			unknown_error ("Cannot get drive from hal");
 		
 		handle_mount (hal_ctx, 
-#ifdef HAVE_POLKIT
-			      pol_ctx, 
-#endif
 			      udi, volume, drive, device, invoked_by_uid, 
 			      invoked_by_syscon_name);
 
